package ee.ut.joshi.connectsix.ai.gametree;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearBoard;

public class NegamaxCached extends NegamaxV3 {

	private static final Logger log = Logger.getLogger( NegamaxCached.class );

	private Map<Integer, Integer> cache = new HashMap<>();

	// private int cachedHits = 0;

	public NegamaxCached( Player plr, Player opp ) {

		super( plr, opp );
	}

	public Integer negamaxRec( Board<? extends Object> board, Player plr, int depth, int alpha, int beta, int color ) {

		if( !( board instanceof LinearBoard ) ) {

			log.fatal( "Cache for board type not supported! Exiting." );
			System.exit( 1 );
		}

		LinearBoard linBoard = (LinearBoard) board;

		Integer cached = cache.get( getCacheKey( linBoard ) );

		if( cached != null ) {

			// log.debug( "Found cached=" + cached + ";cacheKey:" + getCacheKey( linBoard ) );
			// log.debug( "Total cached hits=" + ++cachedHits );

			return cached;
		}

		cached = super.negamaxRec( board, plr, depth, alpha, beta, color );

		// log.info( "Did not find cached=" + cached + ";cacheKey:" + linBoard );

		cache.put( getCacheKey( linBoard ), cached );

		// log.info( "Cache map=" + cache );

		return cached;
	}

	private int getCacheKey( LinearBoard board ) {

		int hash = 9; // arbitrary seed value
		int multiplier = 13; // arbitrary multiplier value

		for( int i = 0; i < board.getFields().length; i++ ) {

			hash = hash * multiplier + board.getFields()[ i ];
		}

		return hash;
	}
}
