package ee.ut.joshi.connectsix.ai.gametree.threaded;

import java.util.Observable;
import java.util.Observer;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.gametree.Negamax;
import ee.ut.joshi.connectsix.ai.gametree.NegamaxV3;
import ee.ut.joshi.connectsix.logic.Competition;
import ee.ut.joshi.connectsix.logic.Trainer;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Player;

public class NegamaxManager implements Observer {

	private static final Logger log = Logger.getLogger( NegamaxManager.class );

	private final int MAXNROFTHREADS = ConnectObotProps.nrOfThreads();

	private static NegamaxManager negamaxManager = new NegamaxManager();

	private boolean threadedActivated = false;

	private int maxUsableThreads = 0;

	private NegamaxManager() {}

	public static NegamaxManager instance() {
		return negamaxManager;
	}

	public Negamax negamaxInstance( Player plr, Player opp ) {

		// Check if there is no running threaded negamax already
		// and we have some spare threads to use //TODO: Add this to properties?
		if( !threadedActivated && maxUsableThreads > 1 ) {

			// We found potential threaded Negamax slot
			// This check checks if we can actually reserve this slot
			if( setThreadedActivated() ) {

				if( log.isDebugEnabled() )
					log.debug( "Returning new ThradedNegamax to use, MAXTHREADS=" + MAXNROFTHREADS + ";usableThreds=" + maxUsableThreads );

				return new ThreadedNegamax( this, plr, opp );
			}
		}

		// Otherwise just return regular Negamax
		return new NegamaxV3( plr, opp );
	}

	public void setThreadedDeactivated() {

		if( log.isDebugEnabled() )
			log.debug( "ThreadedNegamax finished - releasing reserved slot" );

		threadedActivated = false;
	}

	private synchronized boolean setThreadedActivated() {

		// Check if some other thread didn't already set this variable to true
		if( threadedActivated )
			return false;

		// Indicate that we have now running threaded Negamax and other threads should be
		// running the default Negamax without threads
		threadedActivated = true;

		// Show that we successfully managed to reserve threaded Negamax slot
		return true;
	}

	public Integer maxUsableThreads() {

		return maxUsableThreads;
	}

	@Override
	public void update( Observable trainer, Object nrOfRunningGames ) {

		int prevMax = maxUsableThreads;

		if( trainer instanceof Trainer || trainer instanceof Competition ) {

			Integer runningGames = (Integer) nrOfRunningGames;

			// Update number of usable threads + 1 for the dedicated game thread itself
			maxUsableThreads = MAXNROFTHREADS - runningGames + 1;

			if( log.isDebugEnabled() && prevMax < maxUsableThreads )
				log.debug( "New maxUsableThreads=" + maxUsableThreads );
		}
	}
}
