package ee.ut.joshi.connectsix.ai.gametree.threaded;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.gametree.Negamax;
import ee.ut.joshi.connectsix.logic.board.BoardComparator;
import ee.ut.joshi.connectsix.logic.factory.BoardCloneFactory;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;

public class NegamaxThread implements Runnable {

	private static final Logger log = Logger.getLogger( NegamaxThread.class );

	private final ThreadedNegamax negamax;

	private Player player;
	private Player opponent;

	private boolean completed = false;

	private int restultValue;

	private Board<? extends Object> board;
	private Player plr;
	private Integer depth;
	private Integer alpha;
	private Integer beta;
	private Integer color;

	public NegamaxThread( ThreadedNegamax negamax, Player plr, Player opp ) {

		this.negamax = negamax;

		player = plr;
		opponent = opp;
	}

	@Override
	public void run() {

		// log.debug( "NegamaxThread activated" );

		if( beta == null ) {

			log.fatal( "Beta not configured for NegamaxThread!" );
			System.exit( 1 );
		}

		restultValue = -negamaxRec( board, plr, depth, alpha, beta, color );

		// log.debug( "Result from recursive call, restultValue=" + restultValue );

		completed = true;

		negamax.reportResult( this );
	}

	public Integer negamaxRec( Board<? extends Object> board, Player plr, int depth, int alpha, int beta, int color ) {

		if( board.isGameEnd() ) {

			if( board.getWinningPlayer() == player.getValue() )
				return color * Negamax.MAXVALUE;

			if( board.getWinningPlayer() == opponent.getValue() )
				return color * Negamax.MINVALUE;

			return color * negamax.evaluateBoard( board, plr );
		}

		if( depth == Negamax.MAXDEPTH )
			return color * negamax.evaluateBoard( board, plr );

		Player nextPlr = plr == player ? opponent : player;

		String dpth = "negamax-" + nextPlr + "-";

		for( int i = 0; i < depth; i++ ) {
			dpth += "-";
		}

		List<Move> moves = board.getNextMoves();

		if( log.isTraceEnabled() )
			log.trace( dpth + "Next possible moves=" + moves );

		List<Board<? extends Object>> cloneboards = new ArrayList<>();

		for( Move newmove : moves ) {

			Board<? extends Object> cloneBoard = BoardCloneFactory.cloneBoard( board );
			cloneBoard.makeMove( nextPlr, newmove );

			cloneboards.add( cloneBoard );
		}

		if( color == 1 && depth != Negamax.MAXDEPTH - 1 )
			Collections.sort( cloneboards, Collections.reverseOrder( BoardComparator.instance() ) );
		else if( color == -1 && depth != Negamax.MAXDEPTH - 1 )
			Collections.sort( cloneboards, BoardComparator.instance() );

		for( Board<? extends Object> cloneBoard : cloneboards ) {

			Integer value = -1 * negamaxRec( cloneBoard, nextPlr, depth + 1, -beta, -alpha, -color );

			// if( log.isTraceEnabled() )
				// log.trace( dpth + "Result from recursive call, value=" + value + ";move=" + cloneBoard.getLastMove() );

			if( value >= beta ) {

				// log.trace( dpth + "Performing beta cutoff, value=" + value + ";beta=" + beta );

				return value;
			}

			if( value >= alpha ) {

				// log.trace( dpth + "Found better alpha, value=" + value + ";alpha=" + alpha );

				alpha = value;
			}
		}

		return alpha;
	}

	public Board<? extends Object> getBoard() {
		return board;
	}

	public boolean isCompleted() {
		return completed;
	}

	public Integer getResultValue() {
		return restultValue;
	}

	public void setBoard( Board<? extends Object> board ) {
		this.board = board;
	}

	public void setPlr( Player plr ) {
		this.plr = plr;
	}

	public void setDepth( Integer depth ) {
		this.depth = depth;
	}

	public void setAlpha( Integer alpha ) {
		this.alpha = alpha;
	}

	public void setBeta( Integer beta ) {
		this.beta = beta;
	}

	public void setColor( Integer color ) {
		this.color = color;
	}
}
