package ee.ut.joshi.connectsix.ai.genetic;

import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;

/**
 * Individual class kept in genetic algorithm population.<br>
 * - keeps genetic information, i.e RandomForest - keeps value of the individual
 * for natural selection - keeps statistics for logging and nice graphs
 */
public class Individual {

	/** Keeps track of how many generations individual has survived */
	private int generation = 0;

	/** Keeps track of individual value */
	private int value = 0;

	/**
	 * Keeps track of statistics how many games this individual has
	 * won/draw/lost as first and second player
	 */
	private int winsOne = 0;
	private int drwsOne = 0;
	private int lossOne = 0;
	private int winsTwo = 0;
	private int drwsTwo = 0;
	private int lossTwo = 0;

	/** Genetic information, i.e. RandomForest */
	private RandomForest forest;

	/**
	 * Creates new individual with genetic information coming from parameter.
	 * 
	 * @param forest Genetic information for individual
	 */
	public Individual( RandomForest forest ) {

		this.forest = forest;
	}

	/**
	 * Increases generation value for individual.
	 */
	public void incGeneration() {

		generation++;
	}

	public void incWinsOne() {
		winsOne++;
	}

	public void incDrwsOne() {
		drwsOne++;
	}

	public void incLossOne() {
		lossOne++;
	}

	public void incWinsTwo() {
		winsTwo++;
	}

	public void incDrwsTwo() {
		drwsTwo++;
	}

	public void incLossTwo() {
		lossTwo++;
	}

	public RandomForest getForest() {
		return forest;
	}

	public void setForest( RandomForest forest ) {
		this.forest = forest;
	}

	public int getGeneration() {
		return generation;
	}

	public void setGeneration( int generation ) {
		this.generation = generation;
	}

	public int getValue() {
		return value;
	}

	public void setValue( int value ) {
		this.value = value;
	}

	public int getWinsOne() {
		return winsOne;
	}

	public void setWinsOne( int winsOne ) {
		this.winsOne = winsOne;
	}

	public int getDrwsOne() {
		return drwsOne;
	}

	public void setDrwsOne( int drwsOne ) {
		this.drwsOne = drwsOne;
	}

	public int getLossOne() {
		return lossOne;
	}

	public void setLossOne( int lossOne ) {
		this.lossOne = lossOne;
	}

	public int getWinsTwo() {
		return winsTwo;
	}

	public void setWinsTwo( int winsTwo ) {
		this.winsTwo = winsTwo;
	}

	public int getDrwsTwo() {
		return drwsTwo;
	}

	public void setDrwsTwo( int drwsTwo ) {
		this.drwsTwo = drwsTwo;
	}

	public int getLossTwo() {
		return lossTwo;
	}

	public void setLossTwo( int lossTwo ) {
		this.lossTwo = lossTwo;
	}

	@Override
	public String toString() {
		return "Individual [generation=" + generation + ", value=" + value + ", winsOne=" + winsOne + ", drwsOne="
				+ drwsOne + ", lossOne=" + lossOne + ", winsTwo=" + winsTwo + ", drwsTwo=" + drwsTwo + ", lossTwo="
				+ lossTwo + "]";
		// + lossTwo + ", forest=" + forest + "]";
	}
}
