package ee.ut.joshi.connectsix.ai.genetic;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;
import ee.ut.joshi.connectsix.logic.factory.RandomForestParserXML;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

public class InitPopulation {

	private static final Logger log = Logger.getLogger( InitPopulation.class );

	private static final boolean usePrevState = ConnectObotProps.usePreviousState();

	/** Used to train new Random Forests */
	private TrainForest trainForest;

	public InitPopulation( TrainForest trainForest ) {

		this.trainForest = trainForest;
	}

	public void initPopulation( List<Individual> population ) {

		if( usePrevState ) {

			log.info( "Loading previous state from HDD." );

			loadPrevState( population );

		} else {

			log.info( "Generating initial population." );

			generateNewPop( population );
		}
	}

	private void generateNewPop( List<Individual> population ) {

		for( int i = 0; i < Brain.POPULATION; i++ ) {

			Individual genForest = new Individual( trainForest.trainForest() );
			population.add( genForest );
		}
	}

	private void loadPrevState( List<Individual> population ) {

		int iter = findLatestIter();

		if( iter < 2 ) {

			log.fatal( "Did not find enough iterations to restore!" );
			System.exit( 1 );
		}

		String[] iterList = findIterTrees( iter );

		for( String filename : iterList ) {
			
			filename = "log/forests/" + filename;

			log.info( "Found file to parse=" + filename );

			try {

				RandomForestParserXML parser = new RandomForestParserXML( new FileInputStream( filename ) );
				
				Individual genForest = new Individual( parser.parseForest() );
				population.add( genForest );

			} catch( ParserConfigurationException e ) {

				log.error( "Unable to parse tree=" + filename );
				e.printStackTrace();

			} catch( SAXException e ) {

				log.error( "Unable to parse tree=" + filename );
				e.printStackTrace();

			} catch( IOException e ) {

				log.error( "Unable to parse tree=" + filename );
				e.printStackTrace();
				
			} catch( Exception e ) {

				log.error( "Unable to parse tree=" + filename );
				e.printStackTrace();
			}
		}
	}

	private int findLatestIter() {

		int latestIter = 0;

		String latestFile = findLatestFile();

		String[] split = latestFile.split( "_" );

		try {

			latestIter = Integer.parseInt( split[ 2 ] );

		} catch( Exception e ) {

			log.fatal( "Something wrong with tree file names!" );
			e.printStackTrace();

			System.exit( 1 );
		}

		return latestIter;
	}

	private static String findLatestFile() {

		File dir = new File( "log/forests" );

		FilenameFilter filter = new FilenameFilter() {
			public boolean accept( File dir, String name ) {
				return name.startsWith( "forests_iter_" );
			}
		};

		String[] children = dir.list( filter );
		Arrays.sort( children );

		return children[ children.length - 1 ];
	}

	private String[] findIterTrees( int iter ) {

		final int iteration = iter;

		File dir = new File( "log/forests" );

		String[] children = dir.list();

		FilenameFilter filter = new FilenameFilter() {
			public boolean accept( File dir, String name ) {
				return name.startsWith( "forests_iter_" + String.format( "%03d", iteration ) );
			}
		};

		children = dir.list( filter );

		return children;
	}
}
