package ee.ut.joshi.connectsix.ai.genetic.genepool;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.genetic.Brain;
import ee.ut.joshi.connectsix.ai.genetic.Individual;
import ee.ut.joshi.connectsix.ai.randomforest.RandomWoods;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataSet;
import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

public class IndividualRetrainer {

	private static final Logger log = Logger.getLogger( IndividualRetrainer.class );

	private static final int NROFTHREADS = ConnectObotProps.nrOfThreads();

	/** Used to train new Random Forests and Woods */
	private TrainForest trainForest;

	private List<Individual> individuals = new ArrayList<>( Brain.POPULATION );

	public IndividualRetrainer( TrainForest trainForest ) {

		this.trainForest = trainForest;
	}

	public void retrainIndividual( Individual individual ) {

		individuals.add( individual );
	}

	public void startRetraining() {

		log.info( "Started retraining individuals on new data" );

		// Create thread pool for execution
		ExecutorService execSvc = Executors.newFixedThreadPool( NROFTHREADS );
		
		List<RetrainerThread> retrainers = new ArrayList<RetrainerThread>();


		long start = System.currentTimeMillis();
		DataSet dataSet = trainForest.getDataSetDao().getDataSet();
		long end = System.currentTimeMillis();
		log.trace( "Time for DataSet dataSet = trainForest.getDataSetDao().getDataSet();=" + ( end - start ) );

		for( Individual indiv : individuals ) {

			List<RandomWoods> woods = indiv.getForest().getWoods();

			for( int woodsIndx = 0; woodsIndx < woods.size(); woodsIndx++ ) {

				RetrainerThread retrainer = new RetrainerThread( trainForest, dataSet, indiv, woodsIndx );

				retrainers.add( retrainer );

				execSvc.execute( retrainer );
			}
		}

		// Make thread pool closed - no new games added
		execSvc.shutdown();

		log.info( "All retrainings added to threadpool" );

		start = System.currentTimeMillis();

		Runtime runtime = Runtime.getRuntime();

		// Wait until all games have been completed
		while( !execSvc.isTerminated() ) {

			if( System.currentTimeMillis() - start > 1000 * 30 ) {

				log.info( "Memory usage (MB): " + ( ( runtime.totalMemory() - runtime.freeMemory() ) / 1000000 ) );

				start = System.currentTimeMillis();

				logStates( retrainers );
			}

			try {
				Thread.sleep( 1000L );
			} catch( InterruptedException e ) {

				log.fatal( "Thread interrupted! Message=" + e.getMessage() );
				e.printStackTrace();
			}
		}
	}

	private void logStates( List<RetrainerThread> retrainers ) {

		String result = "Current states=";

		Individual prevIndiv = null;
		
		for( RetrainerThread thread : retrainers ) {

			if( prevIndiv != thread.getIndividual() ) {
				result += "\n";
				prevIndiv = thread.getIndividual();
			}
			
			result += thread.getRetrainWoodIndx() + "_";
			result += thread.isFinished() + " ; ";
		}

		log.info( result );
	}
}
