package ee.ut.joshi.connectsix.ai.genetic.genepool;

import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.genetic.Individual;
import ee.ut.joshi.connectsix.ai.randomforest.RandomWoods;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataSet;
import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;

public class RetrainerThread implements Runnable {

	private static final Logger log = Logger.getLogger( RetrainerThread.class );
	
	/** Used to train new Random Forests and Woods */
	private final TrainForest trainForest;

	private final DataSet dataSet;

	private final Individual individual;

	private final int retrainWoodIndx;

	private boolean finished = false;

	public RetrainerThread( TrainForest trainForest, DataSet dataSet, Individual individual, int retrainWoodIndx ) {

		this.trainForest = trainForest;
		this.dataSet = dataSet;
		this.individual = individual;
		this.retrainWoodIndx = retrainWoodIndx;
	}

	@Override
	public void run() {

		List<RandomWoods> woods = individual.getForest().getWoods();

		RandomWoods wood = woods.get( retrainWoodIndx );

		List<Integer> baseAttrs = wood.getBaseAttrs();

		long start = System.currentTimeMillis();

		RandomWoods newWoods = trainForest.trainForestWoods( dataSet, baseAttrs );

		long end = System.currentTimeMillis();
		log.trace( "Time for RandomWoods newWoods = trainForest.trainForestWoods( dataSet, baseAttrs );=" + ( end - start ) );

		woods.set( retrainWoodIndx, newWoods );
		finished = true;

		// log.debug( "Finished training woods for woodIndx=" + retrainWoodIndx );
	}

	public Individual getIndividual() {
		return individual;
	}

	public int getRetrainWoodIndx() {
		return retrainWoodIndx;
	}

	public boolean isFinished() {
		return finished;
	}

	public void setFinished( boolean finished ) {
		this.finished = finished;
	}
}
