package ee.ut.joshi.connectsix.ai.randomforest;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.BasicDataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

public class CacheBuilder {

	/** Defines how many attributes was used to train trees */
	public static final int nrOfAttr = ConnectObotProps.forestNrOfAttrs();

	/**
	 * Generate all possible combinations for input - this is number of attributes to the<br>
	 * power of all possible field states ( player one, player two, empty field );
	 */
	private final List<Byte[]> combinations = new ArrayList<>( 3 ^ nrOfAttr );

	protected HashMap<List<Byte>, DataAttribute> buildCache( ForestTree tree ) {

		final HashMap<List<Byte>, DataAttribute> cachecontent = new HashMap<>();

		Byte[] prefix = new Byte[ 0 ];

		generateCombinations( prefix, tree.getDecisionAttr().size() );

		for( Byte[] attrs : combinations ) {

			DataEntry entry = new BasicDataEntry( tree.getDecisionAttr(), attrs );

			DataAttribute dataAttr = tree.findCategory( entry );

			cachecontent.put( getCacheKey( attrs ), dataAttr );
		}

		return cachecontent;
	}

	private void generateCombinations( Byte[] prefix, int length ) {

		if( length == 0 ) {

			combinations.add( prefix );
			return;
		}

		generateCombinations( concat( prefix, 0 ), length - 1 );
		generateCombinations( concat( prefix, 1 ), length - 1 );
		generateCombinations( concat( prefix, 2 ), length - 1 );
	}

	private Byte[] concat( Byte[] a, int b ) {

		Byte[] c = new Byte[ a.length + 1 ];
		System.arraycopy( a, 0, c, 0, a.length );

		c[ a.length ] = new Byte( (byte) b );

		return c;
	}

	private List<Byte> getCacheKey( Byte[] attrs ) {

		return Arrays.asList( attrs );

		// int hash = 29; // arbitrary seed value
		// int multiplier = 113; // arbitrary multiplier value
		//
		// for( Byte attr : attrs ) {
		//
		// hash = hash * multiplier + attr;
		// }
		//
		// return hash;
	}
}
