package ee.ut.joshi.connectsix.ai.randomforest;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeByte;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;

public class CachedRandomWoods extends RandomWoods {

	private static final Logger log = Logger.getLogger( CachedRandomWoods.class );

	private List<List<Integer>> trainingAttrs;

	private HashMap<List<Byte>, DataAttribute> cachecontent;

	public void buildCache( ForestTree mainTree ) {

		CacheBuilder builder = new CacheBuilder();

		long start = System.currentTimeMillis();
		cachecontent = builder.buildCache( mainTree );
		long end = System.currentTimeMillis();

		log.debug( "Built cached RandomWoods, time taken=" + ( end - start ) );
	}

	@Override
	public Map<DataAttribute, Integer> findCategoryMap( DataEntry dataEntry ) {

		// Cache not built yet or is being built so we can't use it
		if( cachecontent == null ) {

			return super.findCategoryMap( dataEntry );
		}

		Map<DataAttribute, Integer> resultMap = new HashMap<DataAttribute, Integer>();

		for( List<Integer> attrIndxs : trainingAttrs ) {

			DataAttribute cached = cachecontent.get( getCacheKey( dataEntry, attrIndxs ) );

			if( resultMap.containsKey( cached ) ) {

				int count = resultMap.get( cached );

				resultMap.put( cached, count + 1 );

			} else {

				resultMap.put( cached, 1 );
			}
		}

		return resultMap;
	}

	private List<Byte> getCacheKey( DataEntry dataEntry, List<Integer> attrIndxs ) {

		List<Byte> key = new ArrayList<Byte>();

		for( Integer attrIndx : attrIndxs ) {

			DataAttributeByte attrValue = (DataAttributeByte) dataEntry.getAttrValue( attrIndx );

			key.add( attrValue.getValue() );
		}

		return key;

		// int hash = 29; // arbitrary seed value
		// int multiplier = 113; // arbitrary multiplier value
		//
		// for( Integer attrIndx : attrIndxs ) {
		//
		// DataAttributeByte attrValue = (DataAttributeByte) dataEntry.getAttrValue( attrIndx );
		//
		// hash = hash * multiplier + attrValue.getValue();
		// }
		//
		// return hash;
	}

	public List<List<Integer>> getTrainingAttrs() {
		return trainingAttrs;
	}

	public void setTrainingAttrs( List<List<Integer>> trainingAttrs ) {
		this.trainingAttrs = trainingAttrs;
	}

	public String trainingAttrsToString() {

		String result = "\n\t\t<TrainingAttributesList>";

		for( List<Integer> attrIndxs : trainingAttrs ) {

			String attrList = "";

			for( Integer attrIndx : attrIndxs ) {

				attrList += attrIndx + ";";
			}

			result += "\n\t\t\t<TrainingAttributes attributeIndxs=\"" + attrList + "\" />";
		}

		result += "\n\t\t</TrainingAttributesList>";

		return result;
	}

	@Override
	public String toStringXML() {

		String result = "\n\t<RandomWoods>";

		result += trainingAttrsToString();

		// There should be only one tree
		for( ForestTree tree : getTrees() ) {

			result += tree.toStringXML();
		}

		result += "\n\t</RandomWoods>";

		return result;
	}

	@Override
	public void toStringXML( BufferedWriter out ) throws IOException {

		out.write( "\n\t<RandomWoods>" );

		out.write( trainingAttrsToString() );

		// There should be only one tree
		for( ForestTree tree : getTrees() ) {

			tree.toStringXML( out );
		}

		out.write( "\n\t</RandomWoods>" );
	}
}
