package ee.ut.joshi.connectsix.ai.randomforest;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeDraw;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.tree.DecisionTree;

/**
 * A tree as part of the #RandomForest. Extends #DecisionTree by keeping track on which attributes
 * the tree is deciding.
 */
public class ForestTree extends DecisionTree {

	private List<Integer> decisionAttr;

	public ForestTree() {}

	/**
	 * Creates a ForesTree out of a #DecisionTree.
	 * 
	 * @param tree Decision tree which to extend into ForestTree
	 * @param decisionAttr List of attributes that are used in decision tree
	 */
	public ForestTree( DecisionTree tree, List<Integer> decisionAttr ) {

		setRoot( tree.getRoot() );

		this.decisionAttr = decisionAttr;
	}

	@Override
	public DataAttribute findCategory( DataEntry dataEntry ) {

		if( allEmptyFields( dataEntry ) )
			return null;

		return super.findCategory( dataEntry );
	}

	private boolean allEmptyFields( DataEntry dataEntry ) {

		for( Integer attr : decisionAttr ) {

			if( dataEntry.getAttrValue( attr ).compareTo( new DataAttributeDraw() ) != 0 )
				return false;
		}

		return true;
	}

	public List<Integer> getDecisionAttr() {
		return decisionAttr;
	}

	public void setDecisionAttr( List<Integer> decisionAttr ) {
		this.decisionAttr = decisionAttr;
	}

	@Override
	public String toString() {

		return "[ForestTree " + decisionAttr + "]" + super.toString();
	}

	@Override
	public String toStringXML() {

		String attrList = "";

		for( Integer attrIndx : decisionAttr ) {

			attrList += attrIndx + ";";
		}

		return "\n\t\t<ForestTree decisionAttr=\"" + attrList + "\">" + super.toStringXML() + "\t\t</ForestTree>";
	}

	@Override
	public void toStringXML( BufferedWriter out ) throws IOException {

		String attrList = "";

		for( Integer attrIndx : decisionAttr ) {

			attrList += attrIndx + ";";
		}

		out.write( "\n\t\t<ForestTree decisionAttr=\"" + attrList + "\">" );
		super.toStringXML( out );
		out.write( "\t\t</ForestTree>" );
	}
}