package ee.ut.joshi.connectsix.ai.randomforest;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;

/**
 * Random forest is an ensemble classifier that consists of many decision trees
 * and outputs the class that is the mode of the classes output by individual
 * trees.
 */
public class RandomForest {

	private static final Logger log = Logger.getLogger( RandomForest.class );

	/** Individual decision trees */
	private List<RandomWoods> woods = new ArrayList<>();

	/**
	 * @param woods A RandomWoods to be added into forest
	 */
	public void addWoods( RandomWoods woods ) {

		this.woods.add( woods );
	}

	/**
	 * Feeds input to each woods in a forest and collects the category
	 * classification from them. Result is most frequent category as decided by
	 * decision trees.
	 * 
	 * @param dataEntry Data entry on which to find a category
	 * @return Most frequent category decided by decision trees
	 */
	public DataAttribute findCategory( DataEntry dataEntry ) {

		// TODO: Implement
		log.fatal( "FintCategory for RandomForest not implemented!" );
		System.exit( 1 );

		return null;
	}

	/**
	 * Creates a map where keys are category names and value is count of the
	 * category classifications by all trees in each woods.
	 * 
	 * @param dataEntry Data entry on which to find a category
	 * @return Map with category frequency count
	 */
	public Map<DataAttribute, Integer> findCategoryMap( DataEntry dataEntry ) {

		Map<DataAttribute, Integer> resultMap = new HashMap<DataAttribute, Integer>();

		// The result map is sum of resultMaps from all RandomWoods
		for( RandomWoods woods : this.woods ) {

			Map<DataAttribute, Integer> newMap = woods.findCategoryMap( dataEntry );

			addResultMaps( resultMap, newMap );
		}

		if( log.isTraceEnabled() )
			log.trace( "Result from RandomForest=" + resultMap );

		return resultMap;
	}

	private void addResultMaps( Map<DataAttribute, Integer> original, Map<DataAttribute, Integer> newMap ) {

		// Loop through every key in the new map from RandomWoods
		for( Entry<DataAttribute, Integer> entry : newMap.entrySet() ) {

			DataAttribute key = entry.getKey();

			// Check if the key is already present in original map
			if( original.containsKey( key ) ) {

				Integer oldValue = original.get( key );
				Integer newValue = newMap.get( key );

				// If yes, we store the sum of two values as the new value into original map
				original.put( key, oldValue + newValue );

			} else {

				// Otherwise we initialize the key with result from RandomWoods
				original.put( key, newMap.get( key ) );
			}
		}
	}

	public List<RandomWoods> getWoods() {
		return woods;
	}

	public void setWoods( List<RandomWoods> woods ) {
		this.woods = woods;
	}

	@Override
	public String toString() {

		String result = "[RandomForest]\n";

		for( RandomWoods woods : this.woods ) {

			result += woods;
		}

		return result;
	}

	public String toStringXML() {

		String result = "\n<RandomForest>";

		for( RandomWoods woods : this.woods ) {

			result += woods.toStringXML();
		}

		result += "\n</RandomForest>";

		return result;
	}

	public void toStringXML( BufferedWriter out ) throws IOException {

		out.write( "\n<RandomForest>" );

		for( RandomWoods woods : this.woods ) {

			woods.toStringXML( out );
		}

		out.write( "\n</RandomForest>" );
	}
}
