package ee.ut.joshi.connectsix.ai.randomforest;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeLeaf;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeNode;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeRoot;

public class TreeReplicator {

	private static final Logger log = Logger.getLogger( TreeReplicator.class );

	public ForestTree replicateTree( ForestTree tree, List<Integer> newAttrs ) {

		Map<Integer, Integer> replaceMap = new HashMap<Integer, Integer>();

		List<Integer> attrList = tree.getDecisionAttr();

		for( int i = 0; i < attrList.size(); i++ ) {

			replaceMap.put( attrList.get( i ), newAttrs.get( i ) );
		}

		if( log.isTraceEnabled() )
			log.trace( "Replace map=" + replaceMap );

		ForestTree clone = buildForestTree( tree, replaceMap );

		clone.setDecisionAttr( newAttrs );

		return clone;
	}

	private ForestTree buildForestTree( ForestTree origTree, Map<Integer, Integer> replaceMap ) {

		ForestTree tree = new ForestTree();

		TreeRoot root = new TreeRoot();

		tree.setRoot( root );

		// root.setHeaderIndx( Integer.valueOf( rootElem.getAttribute( "headerIndx" ) ) );
		root.setHeaderIndx( replaceMap.get( origTree.getRoot().getHeaderIndx() ) );

		buildSubTree( origTree.getRoot(), root, replaceMap );

		return tree;
	}

	private void buildSubTree( TreeNode original, TreeNode parent, Map<Integer, Integer> replaceMap ) {

		List<TreeNode> nodeList = original.getNodes();

		for( TreeNode origNode : nodeList ) {

			if( origNode instanceof TreeLeaf ) {

				DataAttribute branchAttr = origNode.getBranchValue();
				DataAttribute playerAttr = ( (TreeLeaf) origNode ).getCategoryValue();

				TreeLeaf leaf = new TreeLeaf( parent, playerAttr );

				leaf.setBranchValue( branchAttr );

				parent.addNode( leaf );

			} else if( origNode instanceof TreeNode ) {

				TreeNode child = new TreeNode( parent );

				parent.addNode( child );

				DataAttribute branchValueAttr = origNode.getBranchValue();
				Integer headerIndx = replaceMap.get( origNode.getHeaderIndx() );

				child.setBranchValue( branchValueAttr );
				child.setHeaderIndx( headerIndx );

				buildSubTree( origNode, child, replaceMap );

			}
		}
	}
}
