package ee.ut.joshi.connectsix.ai.randomforest.connect6;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeDraw;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeField;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataSetDao;
import ee.ut.joshi.connectsix.model.Board;

public class Connect6DataSetDao implements DataSetDao {

	private Connection con;

	public Connect6DataSetDao( Connection con ) {

		this.con = con;
	}

	@Override
	public Connect6DataSet getDataSet() {

		Connect6DataSet dataset = new Connect6DataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from connectsix" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				Connect6DataEntry entry = new Connect6DataEntry();

				for( int fieldIndx = 0; fieldIndx < Board.NROFFIELDS; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				byte winner = rs.getByte( Board.NROFFIELDS + 1 );

				if( winner == 0 ) {

					entry.setWinner( new DataAttributeDraw() );
				} else {

					entry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}

	@Override
	public Connect6DataSet getDataSet( Integer limit ) {

		Connect6DataSet dataset = new Connect6DataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			// TODO: SQL exception
			// SELECT TOP 10 * FROM someTable ORDER By NEWID()
			PreparedStatement stmt = con.prepareStatement( "select top " + ( limit * limit )
					+ " * from connectsix order by id desc" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				Connect6DataEntry entry = new Connect6DataEntry();

				for( int fieldIndx = 0; fieldIndx < Board.NROFFIELDS; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				byte winner = rs.getByte( Board.NROFFIELDS + 1 );

				if( winner == 0 ) {

					entry.setWinner( new DataAttributeDraw() );
				} else {

					entry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}

	@Override
	public Connect6DataSet getDataSet( String column, String value ) {

		Connect6DataSet dataset = new Connect6DataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from connectsix where " + column + "=?" );
			stmt.clearParameters();

			stmt.setString( 1, value );

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				Connect6DataEntry entry = new Connect6DataEntry();

				for( int fieldIndx = 0; fieldIndx < Board.NROFFIELDS; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				entry.setWinner( new DataAttributePlayer( rs.getByte( Board.NROFFIELDS + 1 ) ) );

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}
}
