package ee.ut.joshi.connectsix.ai.randomforest.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Scanner;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataEntryDao;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataSet;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataSetDao;

public class ImportDatabase {

	private static final Logger log = Logger.getLogger( ImportDatabase.class );

	private final String exportFrom;
	private final String importTo;

	private Connection exportFromCon;
	private Connection importToCon;

	private LinearDataSetDao exportFromDao;
	private LinearDataEntryDao importToDao;

	private LinearDataSetDao importToSetDao;

	public ImportDatabase( String exportFrom, String importTo ) {

		this.exportFrom = exportFrom;
		this.importTo = importTo;
	}

	public void startImport() {

		initConnections();

		LinearDataSet export = exportFromDao.getDataSet();

		int limit = getConfirmation( export );

		log.info( "Starting import with limit " + limit );

		for( DataEntry entry : exportFromDao.getDataSet( limit ).getData() ) {

			importToDao.saveDataEntry( entry );
		}

		try {
			exportFromCon.close();
			importToCon.close();
		} catch( SQLException e ) {
			e.printStackTrace();
		}

		log.info( "New DB size is " + importToSetDao.getDataSet().getData().size() );
		log.info( "Import successful." );
	}

	private int getConfirmation( LinearDataSet export ) {

		log.info( "Current DB size is " + importToSetDao.getDataSet().getData().size() );
		log.info( "Number of entries to import:" + export.getData().size() );

		log.info( "Confirm export/import [y/n]:" );
		Scanner scn = new Scanner( System.in );

		String answer = scn.nextLine();

		if( !answer.equalsIgnoreCase( "y" ) ) {

			log.info( "Import cancelled, exiting" );
			System.exit( 0 );
		}

		log.info( "Import confirmed. Specify number of entries to import:" );

		int limit = scn.nextInt();

		log.info( "Selected limit is " + limit );

		return limit;
	}

	private void initConnections() {

		try {

			Class.forName( "org.hsqldb.jdbc.JDBCDriver" );
		} catch( ClassNotFoundException e ) {
			e.printStackTrace();
		}

		try {

			exportFromCon = DriverManager.getConnection( "jdbc:hsqldb:import/" + exportFrom, "SA", "" );
			importToCon = DriverManager.getConnection( "jdbc:hsqldb:data/" + importTo, "SA", "" );

		} catch( SQLException e ) {
			e.printStackTrace();
		}

		exportFromDao = new LinearDataSetDao( exportFromCon );
		importToDao = new LinearDataEntryDao( importToCon );
		importToSetDao = new LinearDataSetDao( importToCon );

		log.info( "Connections open:" );
		log.info( "exportFromCon=" + exportFromCon );
		log.info( "importToCon=" + importToCon );
	}
}
