package ee.ut.joshi.connectsix.ai.randomforest.linear;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeDraw;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeField;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataSetDao;

public class LinearDataSetDao implements DataSetDao {

	private Connection con;

	public LinearDataSetDao( Connection con ) {

		this.con = con;
	}

	@Override
	public LinearDataSet getDataSet() {

		LinearDataSet dataset = new LinearDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from linear_smart" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				LinearDataEntry entry = new LinearDataEntry();

				for( int fieldIndx = 0; fieldIndx < InitDatabaseLinear.totalSmartFields; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				byte winner = rs.getByte( InitDatabaseLinear.totalSmartFields + 1 );

				if( winner == 0 ) {

					entry.setWinner( new DataAttributeDraw() );
				} else {

					entry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}

	@Override
	public LinearDataSet getDataSet( Integer limit ) {

		LinearDataSet dataset = new LinearDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			// TODO: SQL exception
			// SELECT TOP 10 * FROM someTable ORDER By NEWID()
			PreparedStatement stmt = con.prepareStatement( "select top " + limit + " * from linear_smart order by id desc" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				LinearDataEntry entry = new LinearDataEntry();

				for( int fieldIndx = 0; fieldIndx < InitDatabaseLinear.totalSmartFields; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				byte winner = rs.getByte( InitDatabaseLinear.totalSmartFields + 1 );

				if( winner == 0 ) {

					entry.setWinner( new DataAttributeDraw() );
				} else {

					entry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}

	@Override
	public LinearDataSet getDataSet( String column, String value ) {

		LinearDataSet dataset = new LinearDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from linear_smart where " + column + "=?" );
			stmt.clearParameters();

			stmt.setString( 1, value );

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				LinearDataEntry entry = new LinearDataEntry();

				for( int fieldIndx = 0; fieldIndx < InitDatabaseLinear.totalSmartFields; fieldIndx++ ) {

					entry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}

				entry.setWinner( new DataAttributePlayer( rs.getByte( InitDatabaseLinear.totalSmartFields + 1 ) ) );

				result.add( entry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}
}
