package ee.ut.joshi.connectsix.ai.randomforest.tictactoe;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeDraw;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeField;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataSetDao;

public class TicDataSetDao implements DataSetDao {

	private Connection con;

	public TicDataSetDao( Connection con ) {

		this.con = con;
	}

	@Override
	public TicDataSet getDataSet() {

		TicDataSet dataset = new TicDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from tictactoe" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				TicDataEntry ticEntry = new TicDataEntry();

				for( int fieldIndx = 0; fieldIndx < 9; fieldIndx++ ) {

					ticEntry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}
				
				byte winner = rs.getByte( 10 );
				
				if( winner == 0 ) {
					
					ticEntry.setWinner( new DataAttributeDraw() );
				} else {
					
					ticEntry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( ticEntry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}

	
	@Override
	public TicDataSet getDataSet( Integer limit ) {

		TicDataSet dataset = new TicDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			// SELECT TOP 10 * FROM someTable ORDER By NEWID() 
			PreparedStatement stmt = con.prepareStatement( "select top " + (limit * limit) + " * from tictactoe order by id desc" );
			stmt.clearParameters();

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				TicDataEntry ticEntry = new TicDataEntry();

				for( int fieldIndx = 0; fieldIndx < 9; fieldIndx++ ) {

					ticEntry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}
				
				byte winner = rs.getByte( 10 );
				
				if( winner == 0 ) {
					
					ticEntry.setWinner( new DataAttributeDraw() );
				} else {
					
					ticEntry.setWinner( new DataAttributePlayer( winner ) );
				}

				result.add( ticEntry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// TODO: Improving possibility
		Collections.shuffle( result );
		
		while( result.size() > limit )
			result.remove( 0 );

		dataset.setData( result );

		return dataset;
	}

	@Override
	public TicDataSet getDataSet( String column, String value ) {

		TicDataSet dataset = new TicDataSet();

		List<DataEntry> result = new ArrayList<>();

		try {

			PreparedStatement stmt = con.prepareStatement( "select * from tictactoe where " + column + "=?" );
			stmt.clearParameters();

			stmt.setString( 1, value );

			ResultSet rs = stmt.executeQuery();

			while( rs.next() ) {

				TicDataEntry ticEntry = new TicDataEntry();

				for( int fieldIndx = 0; fieldIndx < 9; fieldIndx++ ) {

					ticEntry.addFieldValue( fieldIndx, new DataAttributeField( rs.getByte( fieldIndx + 1 ) ) );
				}
				
				ticEntry.setWinner( new DataAttributePlayer( rs.getByte( 10 ) ) );

				result.add( ticEntry );
			}

		} catch( SQLException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dataset.setData( result );

		return dataset;
	}
}
