package ee.ut.joshi.connectsix.ai.randomforest.train;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.db.InitDatabase;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Board;

public class AttributeGenerator {

	private static final Logger log = Logger.getLogger( AttributeGenerator.class );

	/** Defines how many attributes will be used to train trees */
	public static final int nrOfAttr = ConnectObotProps.forestNrOfAttrs();

	public static final int maxAttrRadius = ConnectObotProps.forestAttrRadius();

	public static List<List<Integer>> generateAttributes( List<Integer> mainAttributes ) {

		List<List<Integer>> allAttributes = new ArrayList<>();

		// List<Integer> mainAttributes = getTrainingAttr();
		printPattern( mainAttributes );

		allAttributes.addAll( replicateOverBoard( mainAttributes ) );

		List<Integer> tempAttrs = new ArrayList<>();

		// Vertical plane mirror
		tempAttrs = mirrorHor( mainAttributes );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// Horizontal plane mirror
		tempAttrs = mirrorVer( mainAttributes );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// Horizontal and vertical plane mirror
		tempAttrs = mirrorHor( mainAttributes );
		tempAttrs = mirrorVer( tempAttrs );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// SECOND PART

		// Transposed (or diagonal plane mirror)
		List<Integer> transposed = transpose( mainAttributes );
		printPattern( transposed );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// Vertical plane mirror
		tempAttrs = mirrorHor( transposed );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// Horizontal plane mirror
		tempAttrs = mirrorVer( transposed );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// Horizontal and vertical plane mirror
		tempAttrs = mirrorHor( transposed );
		tempAttrs = mirrorVer( tempAttrs );
		printPattern( tempAttrs );

		allAttributes.addAll( replicateOverBoard( tempAttrs ) );

		// TODO: Remove duplicates

		return allAttributes;
	}

	private static List<Integer> mirrorHor( List<Integer> mainAttributes ) {

		List<Integer> mirrored = new ArrayList<>();

		for( Integer attr : mainAttributes ) {

			int coordX = attr % Board.WIDTH;
			int coordY = (int) Math.floor( attr / Board.WIDTH );

			// Not sure why the -1 is needed, but it works
			int newX = coordX;
			int newY = maxAttrRadius - 1 - coordY;

			Integer mirrorAttr = newX + newY * Board.WIDTH;

			mirrored.add( mirrorAttr );
		}

		return mirrored;
	}

	private static List<Integer> mirrorVer( List<Integer> mainAttributes ) {

		// TODO: Test if this works also for even attribute radius
		List<Integer> mirrored = new ArrayList<>();

		for( Integer attr : mainAttributes ) {

			int coordX = attr % Board.WIDTH;
			int coordY = (int) Math.floor( attr / Board.WIDTH );

			// Not sure why the -1 is needed, but it works
			int newX = maxAttrRadius - 1 - coordX;
			int newY = coordY;

			Integer mirrorAttr = newX + newY * Board.WIDTH;

			mirrored.add( mirrorAttr );
		}

		return mirrored;
	}

	private static List<Integer> transpose( List<Integer> mainAttributes ) {

		// TODO: Test if this works also for even attribute radius
		List<Integer> mirrored = new ArrayList<>();

		for( Integer attr : mainAttributes ) {

			int coordX = attr % Board.WIDTH;
			int coordY = (int) Math.floor( attr / Board.WIDTH );

			// Not sure why the -1 is needed, but it works
			int newX = coordY;
			int newY = coordX;

			Integer mirrorAttr = newX + newY * Board.WIDTH;

			mirrored.add( mirrorAttr );
		}

		return mirrored;
	}

	private static List<List<Integer>> replicateOverBoard( List<Integer> mainAttributes ) {

		List<List<Integer>> replications = new ArrayList<>();

		// Initialize with first index, x-axis
		Integer leftMost = Board.WIDTH;
		Integer rightMost = 0;
		// Initialize with first index, y-axis
		Integer lowest = Board.WIDTH;
		Integer highest = 0;

		// Find attribute with rightmost base index
		for( Integer attr : mainAttributes ) {

			if( attr % Board.WIDTH < leftMost )
				leftMost = attr % Board.WIDTH;

			if( attr % Board.WIDTH > rightMost )
				rightMost = attr % Board.WIDTH;

			if( Math.floor( attr / Board.WIDTH ) < lowest )
				lowest = (int) Math.floor( attr / Board.WIDTH );

			if( Math.floor( attr / Board.WIDTH ) > highest )
				highest = (int) Math.floor( attr / Board.WIDTH );
		}

		// This could be optimized, but there is no point as it is still fast
		for( int x = -Board.WIDTH; x < Board.WIDTH; x++ ) {
			for( int y = -Board.WIDTH; y < Board.WIDTH; y++ ) {

				List<Integer> tempList = new ArrayList<>();

				// Check for out of bounds
				if( x + leftMost < 0 || x + rightMost >= Board.WIDTH )
					continue;

				if( y + lowest < 0 || y + highest >= Board.WIDTH )
					continue;

				// Add new position to temporary list
				for( Integer oldPos : mainAttributes ) {

					tempList.add( oldPos + x + ( y * Board.WIDTH ) );
				}

				replications.add( tempList );
			}
		}

		return replications;
	}

	public static List<Integer> transformToSmart( List<Integer> mainAttributes ) {

		List<Integer> smartFields = new ArrayList<>();

		for( Integer attr : mainAttributes ) {

			int coordX = attr % Board.WIDTH;
			int coordY = (int) Math.floor( attr / Board.WIDTH );

			Integer smartAttr = coordX + coordY * InitDatabase.smartFieldsWidth;

			smartFields.add( smartAttr );
		}

		return smartFields;
	}

	public static List<Integer> getTrainingAttr() {

		List<Integer> trainingAttr = new ArrayList<>();

		Random r = new Random( System.currentTimeMillis() );

		// Find new points until we reach required number
		while( trainingAttr.size() < nrOfAttr ) {

			// End result in linear format [0,..,n), where n is number of fields
			int newpntLinear = 0;

			// TODO: Try with gaussian?
			// int newpntX = (int) Math.round( Math.abs( r.nextGaussian() ) * 2 );
			// int newpntY = (int) Math.round( Math.abs( r.nextGaussian() ) * 2 );

			int newpntX = (int) Math.floor( r.nextDouble() * maxAttrRadius );
			int newpntY = (int) Math.floor( r.nextDouble() * maxAttrRadius );

			// Convert from coordinate system to linear system
			newpntLinear = newpntX;
			newpntLinear = newpntLinear + newpntY * Board.WIDTH;

			// Check if we already have this point
			if( !trainingAttr.contains( newpntLinear ) ) {

				// And some sanity check //TODO: Useless in case of smart fields
				if( newpntLinear < Board.NROFFIELDS )
					trainingAttr.add( newpntLinear );
			}
		}

		log.trace( "Chosen attributes=" + trainingAttr );

		return trainingAttr;
	}

	public static void printPattern( List<Integer> attrList ) {

		if( !log.isTraceEnabled() ) {

			return;
		}

		String result = attrList + "\n";

		for( int f = 0; f < Board.NROFFIELDS; f++ ) {

			int coordX = f % Board.WIDTH;
			int coordY = (int) Math.floor( f / Board.WIDTH );

			if( coordX < maxAttrRadius && coordY < maxAttrRadius ) {

				if( attrList.contains( f ) ) {
					result += "X ";
				} else {
					result += "- ";
					;
				}
			}

			if( f % Board.WIDTH == Board.WIDTH - 1 && coordY < maxAttrRadius )
				result += "\n";
		}

		log.trace( result );
	}
}
