package ee.ut.joshi.connectsix.ai.randomforest.train;

import java.util.HashMap;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataSet;

/**
 * In information theory, entropy is a measure of the uncertainty associated
 * with a random variable. In this context, the term usually refers to the
 * Shannon entropy, which quantifies the expected value of the information
 * contained in a message, usually in units such as bits. In this context, a
 * 'message' means a specific realization of the random variable.
 */
public class TrainEntropy {

	private static final Logger log = Logger.getLogger( TrainEntropy.class );

	/**
	 * Calculates data set entropy based on the categorical value.
	 * 
	 * @param dataSet Data set to be analyzed
	 * @return Entropy value for the data set
	 */
	public Double calcEntropy( DataSet dataSet ) {

		log.trace( "Calculating entropy" );

		// Football example
		// Entropy(S) = - (9/14) Log2 (9/14) - (5/14) Log2 (5/14) = 0.940

		HashMap<DataAttribute, Integer> map = new HashMap<>();

		// Find frequency of each categorical value
		for( DataEntry entry : dataSet.getData() ) {

			DataAttribute category = entry.getCategory();

			if( map.containsKey( category ) ) {

				Integer count = map.get( category );

				map.put( category, count + 1 );

			} else {

				map.put( category, 1 );
			}
		}

		log.trace( map );

		// Entropy(S) = S -p(I) log2 p(I)
		// where p(I) is the proportion of S belonging to class I. S is over c.
		// where log2 = Math.log(x)/Math.log(2);

		Double result = 0.00;

		Integer nrOfEx = dataSet.getData().size();

		log.trace( "Number of total examples: " + nrOfEx );

		for( Integer value : map.values() ) {

			log.trace( "Calculating entoropy on value: " + value );

			result = result - ( (double) value / nrOfEx ) * ( Math.log( (double) value / nrOfEx ) / Math.log( 2 ) );
		}

		log.trace( "Entropy is " + result.toString() );

		return result;
	}
}
