package ee.ut.joshi.connectsix.ai.randomforest.train;

import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.ai.randomforest.RandomWoods;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataSet;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataSetDao;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

public class TrainForest {

	private static final Logger log = Logger.getLogger( TrainForest.class );

	/** Defines how many trees are in the forest */
	public static final int nrOfTrees = ConnectObotProps.forestNrOfTrees();

	/** A single decision tree trainer */
	private TrainWoods trainWoods;

	/** Database connection to get the data information */
	private DataSetDao dataSetDao;

	public TrainForest( DataSetDao dataSetDao ) {

		trainWoods = new TrainWoods( dataSetDao );

		this.dataSetDao = dataSetDao;
	}

	/**
	 * Trains a single ForestTree using DB connection from dataSetDao
	 * 
	 * @return A single trained ForestTree
	 */
	public RandomWoods trainForestWoods() {

		DataSet dataSet = dataSetDao.getDataSet();

		log.debug( "DataSet size=" + dataSet.getData().size() );

		return trainForestWoods( dataSet );
	}

	/**
	 * Trains a single ForestTree using DB connection from dataSetDao based on given attributes.<br>
	 * Can be used to retrain old individuals for example.
	 * 
	 * @return A single trained ForestTree
	 */
	public RandomWoods trainForestWoods( DataSet dataSet, List<Integer> baseAttrs ) {

		log.trace( "DataSet size=" + dataSet.getData().size() );

		return trainWoods.trainWoods( dataSet, baseAttrs );
	}

	/**
	 * Trains full forest using DB connection from dataSetDao.
	 * 
	 * @return Fully trained forest
	 */
	public RandomForest trainForest() {

		DataSet dataSet = dataSetDao.getDataSet();

		return trainForest( dataSet );
	}

	/**
	 * Trains a single ForestTree based on given input data set.
	 * 
	 * @param dataSet Training data for a single tree
	 * @return Fully trained forest tree
	 */
	public RandomWoods trainForestWoods( DataSet dataSet ) {

		return trainWoods.trainWoods( dataSet );
	}

	/**
	 * Trains a RandomForest consisting of ForestTrees
	 * 
	 * @param dataSet The training data set
	 * @return Fully trained RandomForest
	 */
	public RandomForest trainForest( DataSet dataSet ) {

		RandomForest forest = new RandomForest();

		for( int i = 0; i < nrOfTrees; i++ ) {

			RandomWoods woods = trainForestWoods( dataSet );

			forest.addWoods( woods );
		}

		return forest;
	}

	public DataSetDao getDataSetDao() {
		return dataSetDao;
	}
}
