package ee.ut.joshi.connectsix.ai.randomforest.tree;

import java.io.BufferedWriter;
import java.io.IOException;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;

public class DecisionTree {

	private TreeNode root;

	/**
	 * Finds category for a single data entry based on the trained decision values.
	 * 
	 * @param dataEntry Data entry to be evaluated
	 * @return Categorical DataAttribute value of the entry.
	 */
	public DataAttribute findCategory( DataEntry dataEntry ) {

		// if( log.isTraceEnabled() )
		// log.trace( "Looking category for\n" + dataEntry );
		//
		// if( log.isTraceEnabled() )
		// log.trace( "Search tree:" + this );

		DataAttribute result;

		result = findCategoryRec( dataEntry, root );

		// if( log.isTraceEnabled() )
		// log.trace( "Found category " + result + " for\n" + dataEntry );

		return result;
	}

	/**
	 * Recursive call for category decision by going deeper in the decision tree until a #TreeLeaf is found.
	 * 
	 * @param dataEntry Data entry to be evaluated
	 * @param node Starting point of the recursive evaluation
	 * @return Categorical DataAttribute value of the entry.
	 */
	private DataAttribute findCategoryRec( DataEntry dataEntry, TreeNode node ) {

		DataAttribute result = null;

		// if( log.isTraceEnabled() )
		// log.trace( "Looking for category in dataEntry=" + dataEntry );

		// if( log.isTraceEnabled() )
		// log.trace( "Looking for category in node=" + node );

		DataAttribute branchValue = dataEntry.getAttrValue( node.getHeaderIndx() );

		// Find next level tree node that matches the attribute value
		for( TreeNode child : node.getNodes() ) {

			// if( log.isTraceEnabled() )
			// log.trace( "Searching in child\n" + child );

			if( child.getBranchValue().compareTo( branchValue ) == 0 ) {

				// log.trace( "Branch found for branch=" + branchValue );

				if( child instanceof TreeLeaf ) {

					return ( (TreeLeaf) child ).getCategoryValue();
				}

				// TODO: Do something here?
				// else if( child instanceof TreeNodeFailure ) {
				//
				// // log.trace( "Should not be here!" );
				// }

				return findCategoryRec( dataEntry, child );
			}
		}

		// // TODO: In which cases this situation happens?
		// log.trace( "No branch found, something's wrong!" );
		// log.trace( dataEntry );
		// log.trace( "Searching for branch=" + branchValue + ", node.getHeaderIndx=" + node.getHeaderIndx() );

		return result;
	}

	public TreeNode getRoot() {
		return root;
	}

	public void setRoot( TreeNode root ) {
		this.root = root;
	}

	@Override
	public String toString() {

		return "\n" + root.toString();
	}

	public String toStringXML() {

		return "\n\t\t" + root.toStringXML();
	}

	public void toStringXML( BufferedWriter out ) throws IOException {

		out.write( "\n\t\t" );
		root.toStringXML( out );
	}
}
