package ee.ut.joshi.connectsix.ai.randomforest.tree;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;

/**
 * Non terminal node in the decision tree. Has at least one child node with a
 * branch value.
 */
public class TreeNode {

	/** Keeps track of node depth in tree, root is level 1 */
	private int depth;

	/** Parent tree node */
	private TreeNode parent;

	/**
	 * List of child nodes, each of the has unique branch value with possible
	 * values of the attribute
	 */
	private List<TreeNode> nodes = new ArrayList<>();

	/**
	 * Current node branch value - one of the possible attribute values from
	 * parent node
	 */
	private DataAttribute branchValue;

	/** Attribute index that this node decides on */
	private Integer headerIndx;

	public TreeNode() {

	}

	public TreeNode( TreeNode parent ) {

		this.parent = parent;

		// In case parent is RootNode and there is no parent
		this.depth = parent == null ? 0 : parent.getDepth() + 1;
	}

	/**
	 * @param child Child node to be added to child node list
	 */
	public void addNode( TreeNode child ) {

		nodes.add( child );
	}

	public int getDepth() {
		return depth;
	}

	public void setDepth( int depth ) {
		this.depth = depth;
	}

	public TreeNode getParent() {
		return parent;
	}

	public void setParent( TreeNode parent ) {
		this.parent = parent;
	}

	public List<TreeNode> getNodes() {
		return nodes;
	}

	public void setNodes( List<TreeNode> nodes ) {
		this.nodes = nodes;
	}

	public DataAttribute getBranchValue() {
		return branchValue;
	}

	public void setBranchValue( DataAttribute branchValue ) {
		this.branchValue = branchValue;
	}

	public Integer getHeaderIndx() {
		return headerIndx;
	}

	public void setHeaderIndx( Integer headerIndx ) {
		this.headerIndx = headerIndx;
	}

	@Override
	public String toString() {

		String result = new String();

		for( int i = 0; i < depth; i++ ) {
			result += "-";
		}

		result += "TreeNode: branchValue=" + branchValue + ", headerIndx=" + headerIndx;

		for( TreeNode child : nodes ) {
			result += "\n" + child;
		}

		return result;
	}

	public String toStringXML() {

		String result = new String( "\n\t\t" );

		for( int i = 0; i < depth; i++ ) {
			result += "\t";
		}

		result += "<TreeNode branchValue=\"" + branchValue + "\" headerIndx=\"" + headerIndx + "\">";

		for( TreeNode child : nodes ) {
			result += child.toStringXML();
		}

		result += "\n\t\t";

		for( int i = 0; i < depth; i++ ) {
			result += "\t";
		}

		return result + "</TreeNode>";
	}

	public void toStringXML( BufferedWriter out ) throws IOException {

		String result = new String( "\n\t\t" );

		for( int i = 0; i < depth; i++ ) {
			result += "\t";
		}

		out.write( result );
		out.write( "<TreeNode branchValue=\"" + branchValue + "\" headerIndx=\"" + headerIndx + "\">" );

		for( TreeNode child : nodes ) {
			child.toStringXML( out );
		}

		result = "\n\t\t";

		for( int i = 0; i < depth; i++ ) {
			result += "\t";
		}

		out.write( result + "</TreeNode>" );
	}
}
