package ee.ut.joshi.connectsix.logic;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.genetic.Brain;
import ee.ut.joshi.connectsix.ai.genetic.Individual;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataEntryDao;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.linear.SmartDataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.GomokuBoard;
import ee.ut.joshi.connectsix.model.SmartGomokuBoard;

public class DoubleGomokuTrainer extends Trainer {

	private static final Logger log = Logger.getLogger( DoubleGomokuTrainer.class );

	private static final int GAMESPERINDIV = ConnectObotProps.gamesPerIndiv();

	// Threading
	public static final int NROFTHREADS = ConnectObotProps.nrOfThreads();

	public DoubleGomokuTrainer( TrainForest trainForest, DataEntryDao entryDao ) {

		super( trainForest, entryDao );
	}

	@Override
	protected List<GeneticGameController> initGames( List<Individual> pop ) {

		List<GeneticGameController> gameList = new ArrayList<>();

		// Old population vs new population
		for( int oldPop = 0; oldPop < Brain.POPULATION; oldPop++ ) {
			for( int nrofgames = 0; nrofgames < GAMESPERINDIV; nrofgames++ ) {

				Individual indivOne = pop.get( oldPop );
				Individual indivTwo = pop.get( ( oldPop + nrofgames ) % Brain.POPULATION + Brain.POPULATION );

				log.trace( "Mating oldpop " + oldPop + " vs " + ( ( oldPop + nrofgames ) % Brain.POPULATION + Brain.POPULATION ) );

				GeneticGameController game;

				// Log all single moves and first level move scores for first player
				if( oldPop == 0 && nrofgames == 0 )
					game = new GeneticGameController( new SmartGomokuBoard(), indivOne, indivTwo, true );
				else
					game = new GeneticGameController( new SmartGomokuBoard(), indivOne, indivTwo );

				game.logFinalStates();

				gameList.add( game );
			}
		}

		// New population vs old population
		for( int newPop = Brain.POPULATION; newPop < Brain.MAXPOP; newPop++ ) {
			for( int nrofgames = 0; nrofgames < GAMESPERINDIV; nrofgames++ ) {

				Individual indivOne = pop.get( newPop );
				Individual indivTwo = pop.get( ( newPop + nrofgames ) % Brain.POPULATION );

				log.trace( "Mating newpop " + newPop + " vs " + ( nrofgames + newPop ) % Brain.POPULATION );

				GeneticGameController game = new GeneticGameController( new SmartGomokuBoard(), indivOne, indivTwo );

				game.logFinalStates();

				gameList.add( game );
			}
		}

		return gameList;
	}

	@Override
	protected void saveGameState( DataEntryDao entryDao, GeneticGameController game, DataAttributePlayer winnerAttr ) {

		GomokuBoard board = (GomokuBoard) game.getBoard();

		// GomokuDataEntry dataEntry = new GomokuDataEntry();
		LinearDataEntry dataEntry = new SmartDataEntry( board.getFields(), board.getWinningPosition() );

		dataEntry.setWinner( winnerAttr );

		entryDao.saveDataEntry( dataEntry );
	}
}
