package ee.ut.joshi.connectsix.logic;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.genetic.Brain;
import ee.ut.joshi.connectsix.ai.genetic.Individual;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataSet;
import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;

/**
 * Training controller for Football data set example.<br>
 * Does not use MiniMax - only Random Forest and Genetic algorithm
 */
public class FootballTrainer {

	private static final Logger log = Logger.getLogger( FootballTrainer.class );

	/** Number of iterations to train the forests */
	private static final int nrOfIters = 10;

	/** Given data set to be trained on */
	private DataSet dataSet;

	/** Random Forest training controller */
	private TrainForest trainForest;

	public FootballTrainer( DataSet dataSet, TrainForest trainForest ) {

		this.dataSet = dataSet;
		this.trainForest = trainForest;
	}

	/**
	 * Starts training the random forests.
	 */
	public void startTraining() {

		// Genetic algorithm controller
		Brain brain = new Brain( trainForest );

		// Create new random trees on with data from DB.
		brain.initPopulation();

		for( int i = 0; i < nrOfIters; i++ ) {

			log.info( "Training next iteration " + i );

			brain.iteration();

			// Evaluate each individual and determine the value
			for( Individual indiv : brain.getPopulation() ) {
				for( DataEntry entry : dataSet.getData() ) {

					DataAttribute attrValue = indiv.getForest().findCategory( entry );

					if( attrValue != null && attrValue.compareTo( entry.getCategory() ) == 0 ) {

						indiv.setValue( indiv.getValue() + 1 );
					}
				}
			}
		}
	}
}
