package ee.ut.joshi.connectsix.logic.board;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import net.domesdaybook.expression.compiler.sequence.SequenceMatcherCompiler;
import net.domesdaybook.expression.parser.ParseException;
import net.domesdaybook.matcher.sequence.SequenceMatcher;
import net.domesdaybook.reader.ByteArrayReader;
import net.domesdaybook.searcher.Searcher;
import net.domesdaybook.searcher.sequence.BoyerMooreHorspoolSearcher;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.linear.LinearBoard;

public class LinearBoardEvaluator {

	private static final Logger log = Logger.getLogger( LinearBoardEvaluator.class );

	private List<SequenceMatcher> playerOne = new ArrayList<>();
	private List<SequenceMatcher> playerTwo = new ArrayList<>();

	private Map<SequenceMatcher, Integer> scoreMap = new HashMap<>();
	private Map<SequenceMatcher, Integer> sequenceLength = new HashMap<>();

	private static LinearBoardEvaluator instance;

	private LinearBoardEvaluator() {}

	public static LinearBoardEvaluator getInstance() {

		if( instance == null ) {

			instance = new LinearBoardEvaluator();

			try {

				instance.compileSequences();

			} catch( ParseException e ) {

				log.fatal( "Incorrect matcher sequence! Message: " + e.getMessage() );
				e.printStackTrace();
				System.exit( 1 );
			}
		}

		return instance;
	}

	private void compileSequences() throws ParseException {

		// Initiate pattern compiler
		SequenceMatcherCompiler compiler = new SequenceMatcherCompiler();

		Map<String, Integer> scoresPatterns = ScorePatternFactory.generateScorePatterns();

		// Add all patterns and their scores to pre-compiled matcher lists
		for( Entry<String, Integer> entry : scoresPatterns.entrySet() ) {

			String pattern = entry.getKey().replace( "0", "00" ).replace( "X", "01" );

			SequenceMatcher matcher = compiler.compile( pattern );
			playerOne.add( matcher );
			scoreMap.put( matcher, entry.getValue() );
			sequenceLength.put( matcher, entry.getKey().replace( " ", "" ).length() );
		}

		// Add all patterns and their scores to pre-compiled matcher lists
		for( Entry<String, Integer> entry : scoresPatterns.entrySet() ) {

			String pattern = entry.getKey().replace( "0", "00" ).replace( "X", "02" );

			SequenceMatcher matcher = compiler.compile( pattern );
			playerTwo.add( matcher );
			scoreMap.put( matcher, entry.getValue() );
			sequenceLength.put( matcher, entry.getKey().replace( " ", "" ).length() );
		}
	}

	public int boardScore( LinearBoard board ) {

		int result = 0;

		for( SequenceMatcher matcher : playerOne ) {

			int nrOfmatches = searchMatches( matcher, board );

			// log.trace( "Found nrOfMatches=" + nrOfmatches + " for matcher=" + matcher );

			result += nrOfmatches * scoreMap.get( matcher ); // TODO: How to add player value?
		}

		for( SequenceMatcher matcher : playerTwo ) {

			int nrOfmatches = searchMatches( matcher, board );

			// log.trace( "Found nrOfMatches=" + nrOfmatches + " for matcher=" + matcher );

			result -= nrOfmatches * scoreMap.get( matcher ); // TODO: How to add player value?
		}

		return result;
	}

	private int searchMatches( SequenceMatcher matcher, LinearBoard board ) {

		int nrOfMatches = 0;

		Searcher searcher = new BoyerMooreHorspoolSearcher( matcher );

		// long matchIndx = matcher.toRegularExpression( false ).replace( "00", "0" ).length() - 1;
		long matchIndx = sequenceLength.get( matcher ) - 1;

		while( matchIndx != Searcher.NOT_FOUND ) {

			matchIndx = searcher.searchForwards( new ByteArrayReader( board.getFields() ), matchIndx, board.getFields().length - 1 );

			if( matchIndx != Searcher.NOT_FOUND ) {

				// log.info( "Found match at location " + matchIndx + " for " + matcher.toRegularExpression( false ) );

				matchIndx++;
				nrOfMatches++;
			}
		}

		// log.trace( "No more matches for " + matcher.toRegularExpression( false ) );

		return nrOfMatches;
	}
}
