package ee.ut.joshi.connectsix.logic.factory;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.player.HumanPlayer;
import ee.ut.joshi.connectsix.model.player.NegaMaxPlayer;
import ee.ut.joshi.connectsix.model.player.RandomPlayer;
import ee.ut.joshi.connectsix.model.player.ScoreBasedPlayer;

public class PlayerCloneFactory {

	private static final Logger log = Logger.getLogger( PlayerCloneFactory.class );

	public static Player clonePlayer( Player player ) {

		if( player instanceof NegaMaxPlayer )
			return ( (NegaMaxPlayer) player ).clone();

		if( player instanceof RandomPlayer )
			return cloneRandomPlayer( (RandomPlayer) player );

		if( player instanceof ScoreBasedPlayer )
			return cloneScorePlayer( (ScoreBasedPlayer) player );

		if( player instanceof HumanPlayer )
			return cloneHumanPlayer( (HumanPlayer) player );

		log.debug( "Cloning player not supported, player=" + player );
		System.exit( 1 );

		return null;
	}

	private static Player cloneRandomPlayer( RandomPlayer player ) {

		RandomPlayer clone = new RandomPlayer( player.getValue() );
		clone.setName( player.getName() );
		clone.setRandom( player.getRandom() );

		return clone;
	}

	private static Player cloneScorePlayer( ScoreBasedPlayer player ) {

		ScoreBasedPlayer clone = new ScoreBasedPlayer( player.getValue() );
		clone.setName( player.getName() );

		return clone;
	}

	private static HumanPlayer cloneHumanPlayer( HumanPlayer player ) {

		HumanPlayer clone = new HumanPlayer( player.getValue(), player.getName() );

		return clone;
	}
}
