package ee.ut.joshi.connectsix.logic.factory;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.randomforest.CachedRandomWoods;
import ee.ut.joshi.connectsix.ai.randomforest.ForestTree;
import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeField;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeLeaf;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeNode;
import ee.ut.joshi.connectsix.ai.randomforest.tree.TreeRoot;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

/**
 * Parses XML forests to be used in application.
 */
public class RandomForestParserXML {

	private static final Logger log = Logger.getLogger( RandomForestParserXML.class );

	/** Defines how many trees are in the forest */
	public static final int nrOfTrees = ConnectObotProps.forestNrOfTrees();

	private final Document doc;

	public RandomForestParserXML( InputStream io ) throws ParserConfigurationException, SAXException, IOException {

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		doc = db.parse( io );
		doc.getDocumentElement().normalize();
	}

	public RandomForest parseForest() {

		RandomForest forest = new RandomForest();

		NodeList woodsNodes = doc.getElementsByTagName( "RandomWoods" );

		log.debug( "woodNodes length=" + woodsNodes.getLength() );

		for( int woodsIndx = 0; woodsIndx < woodsNodes.getLength() && woodsIndx < nrOfTrees; woodsIndx++ ) {

			CachedRandomWoods woods = new CachedRandomWoods();

			Element woodsElem = (Element) woodsNodes.item( woodsIndx );

			List<List<Integer>> trainingAttrs = findTrainingAttrs( woodsElem );

			woods.setTrainingAttrs( trainingAttrs );

			NodeList treeNodes = woodsElem.getElementsByTagName( "ForestTree" );
			if( treeNodes.getLength() > 1 ) {

				log.fatal( "Too many ForestTrees under RandomWoods" );
				System.exit( 0 );
			}

			Element treeElem = (Element) treeNodes.item( 0 );

			Scanner scn = new Scanner( treeElem.getAttribute( "decisionAttr" ) );
			scn.useDelimiter( ";" );

			List<Integer> decisionAttr = new ArrayList<>();

			while( scn.hasNextInt() ) {

				decisionAttr.add( scn.nextInt() );
			}

			log.debug( decisionAttr );

			TreeRoot root = new TreeRoot();

			ForestTree tree = new ForestTree();

			tree.setRoot( root );
			tree.setDecisionAttr( decisionAttr );

			woods.addTree( tree );
			woods.setBaseAttrs( decisionAttr );

			Element rootElem = (Element) treeElem.getElementsByTagName( "TreeRoot" ).item( 0 );

			root.setHeaderIndx( Integer.valueOf( rootElem.getAttribute( "headerIndx" ) ) );

			buildSubTree( rootElem, root );

			woods.buildCache( tree );

			forest.addWoods( woods );
		}

		return forest;
	}

	private List<List<Integer>> findTrainingAttrs( Element woodsElem ) {

		List<List<Integer>> result = new ArrayList<>();

		NodeList attrList = woodsElem.getElementsByTagName( "TrainingAttributesList" );

		if( attrList.getLength() > 1 ) {

			log.fatal( "Too many TrainingAttributesList under RandomWoods" );
			System.exit( 0 );
		}

		Element trainingAttributesList = (Element) attrList.item( 0 );

		NodeList attrNodes = trainingAttributesList.getElementsByTagName( "TrainingAttributes" );

		for( int treeIndx = 0; treeIndx < attrNodes.getLength(); treeIndx++ ) {

			Element attrIndxsElem = (Element) attrNodes.item( treeIndx );

			Scanner scn = new Scanner( attrIndxsElem.getAttribute( "attributeIndxs" ) );
			scn.useDelimiter( ";" );

			List<Integer> decisionAttr = new ArrayList<>();

			while( scn.hasNextInt() ) {

				decisionAttr.add( scn.nextInt() );
			}

			result.add( decisionAttr );
		}

		return result;
	}

	private void buildSubTree( Element rootElem, TreeNode parent ) {

		rootElem.normalize();

		NodeList nodeList = rootElem.getChildNodes();

		for( int i = 0; i < nodeList.getLength(); i++ ) {

			Node temp = nodeList.item( i );

			if( temp.getNodeType() != Node.ELEMENT_NODE )
				continue;

			Element element = (Element) temp;

			if( element.getNodeName().matches( "TreeNode" ) ) {

				TreeNode child = new TreeNode( parent );

				parent.addNode( child );

				Byte branchValue = Byte.valueOf( element.getAttribute( "branchValue" ) );
				Integer headerIndx = Integer.valueOf( element.getAttribute( "headerIndx" ) );

				DataAttribute branchValueAttr = new DataAttributeField( branchValue );

				child.setBranchValue( branchValueAttr );
				child.setHeaderIndx( headerIndx );

				buildSubTree( element, child );

			} else if( element.getNodeName().matches( "TreeLeaf" ) ) {

				Byte branchValue = Byte.parseByte( element.getAttribute( "branchValue" ) );
				Byte categoryValue = Byte.parseByte( element.getAttribute( "categoryValue" ) );

				DataAttribute branchAttr = new DataAttributeField( branchValue );
				DataAttribute playerAttr = new DataAttributePlayer( categoryValue );

				TreeLeaf leaf = new TreeLeaf( parent, playerAttr );

				leaf.setBranchValue( branchAttr );

				parent.addNode( leaf );

			}
		}
	}
}
