package ee.ut.joshi.connectsix.logic.factory;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.CachedC6Board;
import ee.ut.joshi.connectsix.model.CachedGomokuBoard;
import ee.ut.joshi.connectsix.model.ConnectFourBoard;
import ee.ut.joshi.connectsix.model.SmartC6Board;
import ee.ut.joshi.connectsix.model.SmartGomokuBoard;
import ee.ut.joshi.connectsix.model.TicBoard;

public class ReverseBoardFactory {

	private static final Logger log = Logger.getLogger( ReverseBoardFactory.class );

	public static Board<? extends Object> reverseBoard( Board<? extends Object> board ) {

		Board<? extends Object> cloneBoard = BoardCloneFactory.cloneBoard( board );

		if( cloneBoard instanceof TicBoard )
			return reverse( (TicBoard) cloneBoard );
		else if( cloneBoard instanceof CachedC6Board )
			return reverse( (CachedC6Board) cloneBoard );
		else if( cloneBoard instanceof SmartC6Board )
			return reverse( (SmartC6Board) cloneBoard );
		else if( cloneBoard instanceof SmartGomokuBoard )
			return reverse( (SmartGomokuBoard) cloneBoard );
		else if( cloneBoard instanceof CachedGomokuBoard )
			return reverse( (CachedGomokuBoard) cloneBoard );
		else if( cloneBoard instanceof ConnectFourBoard )
			return reverse( (ConnectFourBoard) cloneBoard );
		// else if( board instanceof MatrixBoard )
		// return cloneMatrixBoard( (MatrixBoard) board );
		// else if( board instanceof LinearBoard )
		// return cloneLinearBoard( (LinearBoard) board );

		log.fatal( "Reversing board not supported." );
		System.exit( 1 );

		return null;
	}

	private static Board<? extends Object> reverse( TicBoard board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}

	private static SmartC6Board reverse( SmartC6Board board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}

	private static ConnectFourBoard reverse( ConnectFourBoard board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}

	private static CachedC6Board reverse( CachedC6Board board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}

	private static SmartGomokuBoard reverse( SmartGomokuBoard board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}

	private static CachedGomokuBoard reverse( CachedGomokuBoard board ) {

		byte[] fields = board.getFields();

		for( int i = 0; i < fields.length; i++ ) {

			if( fields[ i ] == Board.EMPTY )
				continue;
			else if( fields[ i ] == Board.PLRONE )
				fields[ i ] = Board.PLRTWO;
			else
				fields[ i ] = Board.PLRONE;
		}

		board.setFields( fields );

		return board;
	}
}
