package ee.ut.joshi.connectsix.logic.factory;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.randomforest.db.InitDatabase;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Position;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;

public class SmartFieldsExtractor {

	private static final Logger log = Logger.getLogger( SmartFieldsExtractor.class );

	private static int attrRadius = (int) Math.ceil( ConnectObotProps.forestAttrRadius() / 2 );

	public static byte[] extractSmartFields( byte[] fields, Position winPos ) {

		if( InitDatabase.smartFieldsWidth % 2 == 0 ) {

			log.fatal( "Smart fields width is not odd, width=" + InitDatabase.smartFieldsWidth + ". Exiting." );
			System.exit( 1 );
		}

		if( winPos instanceof LinearPosition ) {

			return linearPosition( fields, (LinearPosition) winPos );
		} else {

			log.fatal( "Unable to extract position=" + winPos );
			System.exit( 1 );
		}

		return null;
	}

	private static byte[] linearPosition( byte[] fields, LinearPosition winPos ) {

		byte[] smartFields = new byte[ InitDatabase.totalSmartFields ];

		int winIndx = winPos.getPosition();

		// If the win position is near border, shift it to be more in the center
		winIndx = adjustWinIndx( winIndx );

		// Find all smart fields box borders
		int borderLeft = getBorderLeft( winIndx );
		int borderRight = getBorderRight( winIndx );
		int borderTop = getBorderTop( winIndx );
		int borderBot = getBorderBot( winIndx );

		// For tracking smart board index, we use s index and increase it each time we find smart field
		for( int s = 0, f = 0; f < Board.NROFFIELDS; f++ ) {

			int iMod = f % Board.WIDTH;

			if( borderLeft <= iMod && iMod <= borderRight && borderTop <= f && f <= borderBot )
				smartFields[ s++ ] = fields[ f ];
		}

		return smartFields;
	}

	private static int adjustWinIndx( int winIndx ) {

		log.trace( "adjustWinIndx, winIndx=" + winIndx );

		int winIndxCol = winIndx % Board.WIDTH;
		int winIndxRow = (int) Math.floor( winIndx / Board.WIDTH );

		// If we are too left, adjust win index to be more right
		if( winIndxCol < attrRadius )
			winIndxCol = attrRadius;

		// If we are too right, adjust win index to be more left
		if( winIndxCol > Board.WIDTH - attrRadius - 1 )
			winIndxCol = Board.WIDTH - attrRadius - 1;

		// If we are too high, adjust win index to be more lower
		if( winIndxRow < attrRadius )
			winIndxRow = attrRadius;

		// If we are too low, adjust win index to be more higher
		if( winIndxRow > Board.WIDTH - attrRadius - 1 )
			winIndxRow = Board.WIDTH - attrRadius - 1;

		// Calculate new win position
		int newWinIndx = Board.WIDTH * winIndxRow + winIndxCol;

		return newWinIndx;
	}

	private static int getBorderLeft( int winIndx ) {

		int result = winIndx - attrRadius;

		result = result % Board.WIDTH;

		return result;
	}

	private static int getBorderRight( int winIndx ) {

		int result = winIndx + attrRadius;

		result = result % Board.WIDTH;

		return result;
	}

	private static int getBorderTop( int winIndx ) {

		int row = winIndx - Board.WIDTH * attrRadius;

		int rowBase = (int) Math.floor( row / Board.WIDTH ) * Board.WIDTH;

		int result = rowBase + getBorderLeft( winIndx );

		return result;
	}

	private static int getBorderBot( int winIndx ) {

		int row = winIndx + Board.WIDTH * attrRadius;

		int rowBase = (int) Math.floor( row / Board.WIDTH ) * Board.WIDTH;

		int result = rowBase + getBorderRight( winIndx );

		return result;
	}
}
