package ee.ut.joshi.connectsix.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.logic.Competition;
import ee.ut.joshi.connectsix.logic.factory.RandomForestParserXML;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.player.NegaMaxPlayer;

public class CompetitionTest {

	private static final Logger log = Logger.getLogger( HumanAITest.class );

	private static final String PATH = "test/competition/";

	public static void main( String[] args ) throws ParserConfigurationException, SAXException, IOException, InterruptedException {

		PropertyConfigurator.configure( "etc/log4j.properties" );
		ConnectObotProps props = new ConnectObotProps();
		props.load( new FileInputStream( "etc/gomoku.properties" ) );

		String[] foresFiles = findForestFiles();

		Arrays.sort( foresFiles );

		List<RandomForest> forests = parseForestFiles( foresFiles );

		List<Player> players = new ArrayList<>();

		for( RandomForest forest : forests ) {

			NegaMaxPlayer player = new NegaMaxPlayer( 0, forest );
			// Set name by IP address
			player.setName( foresFiles[ forests.indexOf( forest ) ].split( "_" )[ 1 ] );

			players.add( player );
		}

		Competition comp = new Competition( players );

		Thread compThread = new Thread( comp, "competition" );
		compThread.setDaemon( true );

		compThread.start();

		while( compThread.isAlive() ) {

			Thread.sleep( 10000 );
		}

		log.info( "Competition complete!" );
	}

	private static List<RandomForest> parseForestFiles( String[] foresFiles ) throws ParserConfigurationException, SAXException,
			IOException {

		List<RandomForest> forests = new ArrayList<>();

		InputStream io;

		for( String file : foresFiles ) {

			log.info( "Forest file=" + file );

			io = new FileInputStream( PATH + file );

			RandomForest forest = new RandomForestParserXML( io ).parseForest();

			forests.add( forest );
		}

		if( forests.size() < 2 ) {

			System.out.println( "Not enough forests found in test/competition folder. Add forests to start competition" );
			System.exit( 0 );
		}

		return forests;

	}

	private static String[] findForestFiles() {

		File dir = new File( PATH );

		String[] children = dir.list();

		FilenameFilter filter = new FilenameFilter() {
			public boolean accept( File dir, String name ) {
				return name.startsWith( "forest" );
			}
		};

		children = dir.list( filter );

		return children;
	}
}
