package ee.ut.joshi.connectsix.main.fourinarow;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.gametree.threaded.NegamaxManager;
import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.logic.Competition;
import ee.ut.joshi.connectsix.logic.GameController;
import ee.ut.joshi.connectsix.logic.factory.RandomForestParserXML;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.ConnectFourBoard;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.player.DrawPlayer;
import ee.ut.joshi.connectsix.model.player.HumanPlayer;
import ee.ut.joshi.connectsix.model.player.MiniMaxPlayer;
import ee.ut.joshi.connectsix.model.player.NegaMaxPlayer;
import ee.ut.joshi.connectsix.model.player.RandomPlayer;

public class ConnectFourAITest {

	private static final Logger log = Logger.getLogger( ConnectFourAITest.class );

	public static void main( String[] args ) throws ParserConfigurationException, SAXException, IOException {

		PropertyConfigurator.configure( "etc/log4j-aitest.properties" );
		ConnectObotProps props = new ConnectObotProps();
		props.load( new FileInputStream( "etc/connect-four.properties" ) );

		// TODO: Add to properties
		String filename = "test/humanaitest.xml";

		// TODO: Quick hack to make negamax work in threads
		NegamaxManager.instance().update( new Competition( null ), 1 );

		InputStream io;

		try {
			io = new FileInputStream( filename );
		} catch( FileNotFoundException e ) {

			System.out.println( "File not found: " + filename );
			return;
		}

		log.info( "Forest file=" + filename );

		RandomForest forest;
		RandomForestParserXML parser = new RandomForestParserXML( io );

		forest = parser.parseForest();

		if( args.length != 1 ) {

			System.out.println( "Usage: human | computer | random | AIvsAI" );
			System.exit( 1 );
		}

		if( args[ 0 ].compareTo( "human" ) == 0 )
			humanStarts( forest );

		if( args[ 0 ].compareTo( "computer" ) == 0 )
			computerStarts( forest );

		if( args[ 0 ].compareTo( "random" ) == 0 ) {
			randomTestsAI( forest, 10 );
			randomTestsRandom( forest, 10 );
		}

		if( args[ 0 ].compareTo( "AIvsAI" ) == 0 ) {
			testAIvsAI( forest );
		}
	}

	private static void humanStarts( RandomForest forest ) {

		HumanPlayer plrOne = new HumanPlayer( Board.PLRONE, "Kalle" );
		NegaMaxPlayer plrTwo = new NegaMaxPlayer( Board.PLRTWO, forest, true );

		GameController game = new GameController( new ConnectFourBoard(), plrOne, plrTwo );
		game.setActivePlayer( plrOne );

		game.logAllMoves();
		game.logFinalStates();

		game.startGame();

		System.out.println( game.getBoard() );
		System.out.println( "Winner: " + game.getWinner() );
	}

	private static void computerStarts( RandomForest forest ) {

		NegaMaxPlayer plrOne = new NegaMaxPlayer( Board.PLRONE, forest, true );
		HumanPlayer plrTwo = new HumanPlayer( Board.PLRTWO, "Kalle" );

		GameController game = new GameController( new ConnectFourBoard(), plrOne, plrTwo );
		game.setActivePlayer( plrOne );

		game.logAllMoves();
		game.logFinalStates();

		game.startGame();

		System.out.println( game.getBoard() );
		System.out.println( "Winner: " + game.getWinner() );
	}

	private static void randomTestsAI( RandomForest forest, int games ) {

		MiniMaxPlayer plrOne = new NegaMaxPlayer( Board.PLRONE, forest, true );
		RandomPlayer plrTwo = new RandomPlayer( Board.PLRTWO );

		Map<Player, Integer> resultMap = new HashMap<Player, Integer>();

		resultMap.put( plrOne, 0 );
		resultMap.put( plrTwo, 0 );
		resultMap.put( new DrawPlayer(), 0 );

		for( int i = 0; i < games; i++ ) {

			GameController game = new GameController( new ConnectFourBoard(), plrOne, plrTwo );
			game.setActivePlayer( plrOne );

			game.logAllMoves();
			game.logFinalStates();

			game.startGame();

			// if( game.getWinner().equals( plrTwo ) )
			// System.out.println( game.getBoard() );

			resultMap.put( game.getWinner(), resultMap.get( game.getWinner() ) + 1 );
		}

		System.out.println( resultMap );
	}

	private static void randomTestsRandom( RandomForest forest, int games ) {

		RandomPlayer plrOne = new RandomPlayer( (int) Board.PLRONE );

		MiniMaxPlayer plrTwo = new NegaMaxPlayer( (int) Board.PLRTWO );
		plrTwo.setForest( forest );

		Map<Player, Integer> resultMap = new HashMap<Player, Integer>();

		resultMap.put( plrOne, 0 );
		resultMap.put( plrTwo, 0 );
		resultMap.put( new DrawPlayer(), 0 );

		for( int i = 0; i < games; i++ ) {

			GameController game = new GameController( new ConnectFourBoard(), plrOne, plrTwo );
			game.setActivePlayer( plrOne );

			game.logAllMoves();
			game.logFinalStates();

			game.startGame();

			// if( game.getWinner().equals( plrOne ) )
			// System.out.println( game.getBoard() );

			resultMap.put( game.getWinner(), resultMap.get( game.getWinner() ) + 1 );
		}

		System.out.println( resultMap );
	}

	private static void testAIvsAI( RandomForest forest ) {

		NegaMaxPlayer plrOne = new NegaMaxPlayer( Board.PLRONE );
		plrOne.setForest( forest );
		NegaMaxPlayer plrTwo = new NegaMaxPlayer( Board.PLRTWO );
		plrTwo.setForest( forest );

		GameController game = new GameController( new ConnectFourBoard(), plrOne, plrTwo );
		game.setActivePlayer( plrOne );

		game.logAllMoves();
		game.logFinalStates();

		game.startGame();

		System.out.println( game.getBoard() );
		System.out.println( "Winner: " + game.getWinner() );
	}

}
