package ee.ut.joshi.connectsix.main.fourinarow;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import ee.ut.joshi.connectsix.ai.genetic.Brain;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataEntryDao;
import ee.ut.joshi.connectsix.ai.randomforest.db.DataSetDao;
import ee.ut.joshi.connectsix.ai.randomforest.db.InitDatabase;
import ee.ut.joshi.connectsix.ai.randomforest.linear.InitDatabaseLinear;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataEntryDao;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataSetDao;
import ee.ut.joshi.connectsix.ai.randomforest.train.TrainForest;
import ee.ut.joshi.connectsix.logic.ConnectFourTrainer;
import ee.ut.joshi.connectsix.logic.TicTrainer;
import ee.ut.joshi.connectsix.logic.Trainer;
import ee.ut.joshi.connectsix.logic.factory.RandomGamesGenerator;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.ConnectFourBoard;

public class ConnectFourTraining {

	private static final Logger log = Logger.getLogger( ConnectFourTraining.class );

	public static long TIMELIMIT; // loaded from properties
	private static long STARTTIME = System.currentTimeMillis();

	public static void main( String[] args ) {

		try {

			startProcess();
		} catch( FileNotFoundException e ) {

			log.fatal( e.getMessage() );

			e.printStackTrace();
			System.exit( 1 );
		} catch( IOException e ) {

			log.fatal( e.getMessage() );

			e.printStackTrace();
			System.exit( 1 );
		} catch( InterruptedException e ) {

			log.fatal( e.getMessage() );

			e.printStackTrace();
			System.exit( 1 );
		} catch( OutOfMemoryError e ) {

			log.error( "Out of memory exception:" + e.getMessage() );
			e.printStackTrace();
			System.exit( 1 );
		}
	}

	public static void startProcess() throws InterruptedException, FileNotFoundException, IOException {

		PropertyConfigurator.configure( "etc/log4j.properties" );

		ConnectObotProps props = new ConnectObotProps();
		props.load( new FileInputStream( "etc/connect-four.properties" ) );

		TIMELIMIT = ConnectObotProps.timeLimit();

		log.debug( props );

		log.info( "Configuration:" );
		log.info( "Number of threads in pool=" + TicTrainer.NROFTHREADS );
		log.info( "Population=" + Brain.POPULATION );
		log.info( "Mutation=" + Brain.MUTATION );
		log.info( "Nr of trees=" + TrainForest.nrOfTrees );
		log.info( "WIN1=" + TicTrainer.WINVALUE );
		log.info( "DRW1=" + TicTrainer.DRAWVALUE );
		log.info( "LOS1=" + TicTrainer.LOSSVALUE );
		log.info( "WIN2=" + TicTrainer.WINVALUE2 );
		log.info( "DRW2=" + TicTrainer.DRAWVALUE2 );
		log.info( "LOS2=" + TicTrainer.LOSSVALUE2 );

		InitDatabase db = new InitDatabaseLinear();

		DataEntryDao entryDao = new LinearDataEntryDao( db.getConnection() );

		if( !ConnectObotProps.usePreviousState() ) {
			log.info( "Creating DB of random games" );

			RandomGamesGenerator.playScoredVsRandom( entryDao, new ConnectFourBoard(), 10 );
			log.info( "Scored VS Random games complete" );
			RandomGamesGenerator.playRandomVsScored( entryDao, new ConnectFourBoard(), 10 );
			log.info( "Random VS Scored games complete" );
			RandomGamesGenerator.playRandomVsRandom( entryDao, new ConnectFourBoard(), 200 );
			log.info( "Random VS Random games complete" );
		}

		DataSetDao setDao = new LinearDataSetDao( db.getConnection() );

		TrainForest trainForest = new TrainForest( setDao );

		Trainer trainer = new ConnectFourTrainer( trainForest, entryDao );

		Thread trainerThread = new Thread( trainer, "trainer" );
		trainerThread.setDaemon( true );

		trainerThread.start();

		while( trainerThread.isAlive() && System.currentTimeMillis() - STARTTIME < TIMELIMIT ) {

			Thread.sleep( 10000 );
		}

		if( trainerThread.isAlive() )
			trainerThread.interrupt();

		log.info( "Time is up or training complete, exiting" );

		System.exit( 0 );
	}
}
