package ee.ut.joshi.connectsix.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.linear.LinearDualMove;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;

public class CachedC6Board extends Connect6Board {

	private static final Logger log = Logger.getLogger( CachedC6Board.class );

	private List<Move> cachedMoves;

	public CachedC6Board() {

		this( true );
	}

	public CachedC6Board( boolean init ) {

		// TODO: Test speed for this.
		if( init ) {

			byte[] fields = new byte[Board.NROFFIELDS];

			for( int i = 0; i < fields.length; i++ ) {

				fields[ i ] = 0;
			}

			// According to rules, second player starts in the middle
			// with a single move (or first.. question of definition)
			fields[ Board.NROFFIELDS / 2 ] = Board.PLRTWO;

			setFields( fields );
		}
	}

	@Override
	public void makeMove( Player player, Move move ) {

		log.trace( "Making move=" + move + ";player=" + player );
		log.trace( "Current cached move list=" + cachedMoves );

		super.makeMove( player, move );

		DualMove dualMove = (DualMove) move;

		LinearPosition posOne = (LinearPosition) dualMove.getFstPos();
		LinearPosition posTwo = (LinearPosition) dualMove.getSndPos();

		List<Move> removeList = new ArrayList<>();

		for( Move tmpMove : cachedMoves ) {

			DualMove tmpDual = (DualMove) tmpMove;

			LinearPosition tmpOne = (LinearPosition) tmpDual.getFstPos();
			LinearPosition tmpTwo = (LinearPosition) tmpDual.getSndPos();

			if( posOne.getPosition() == tmpOne.getPosition() || posTwo.getPosition() == tmpOne.getPosition() )
				removeList.add( tmpMove );

			if( posOne.getPosition() == tmpTwo.getPosition() || posTwo.getPosition() == tmpTwo.getPosition() )
				removeList.add( tmpMove );
		}

		cachedMoves.removeAll( removeList );

		log.trace( "New cached move list=" + cachedMoves );
	}

	@Override
	public List<Move> getNextMoves() {

		if( cachedMoves == null ) {

			preCacheMoves();
		}

		return cachedMoves;
	}

	private void preCacheMoves() {

		cachedMoves = new ArrayList<Move>( Board.NROFFIELDS * Board.NROFFIELDS );

		// First loop
		for( int positionOne = 0; positionOne < NROFFIELDS; positionOne++ ) {

			// Check if first position is legal move
			if( getFields()[ positionOne ] == 0 ) {

				// Second loop starting from next position
				for( int positionTwo = positionOne + 1; positionTwo < NROFFIELDS; positionTwo++ ) {

					// Check if second position is legal move
					if( getFields()[ positionTwo ] == 0 ) {

						LinearPosition fstPos = new LinearPosition( positionOne );
						LinearPosition sndPos = new LinearPosition( positionTwo );

						// Create new legal move and add to list
						cachedMoves.add( new LinearDualMove( fstPos, sndPos ) );
					}
				}
			}
		}
	}

	public void setCachedMoves( List<Move> cachedMoves ) {
		this.cachedMoves = cachedMoves;
	}

	public List<Move> getCachedMoves() {
		return cachedMoves;
	}
}
