package ee.ut.joshi.connectsix.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.model.linear.LinearMove;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;

public class CachedGomokuBoard extends GomokuBoard {

	private static final Logger log = Logger.getLogger( CachedGomokuBoard.class );

	private List<Move> cachedMoves;

	public CachedGomokuBoard() {

		this( true );
		preCacheMoves();
	}

	public CachedGomokuBoard( boolean init ) {

		if( init ) {

			byte[] fields = new byte[Board.NROFFIELDS];

			for( int i = 0; i < fields.length; i++ ) {

				fields[ i ] = 0;
			}

			setFields( fields );
		}
	}

	@Override
	public void makeMove( Player player, Move move ) {

		if( log.isTraceEnabled() ) {

			log.trace( "Making move=" + move + ";player=" + player );
			log.trace( "Current cached move list=" + cachedMoves );
		}

		super.makeMove( player, move );

		LinearPosition position = (LinearPosition) move.getPosition();

		List<Move> removeList = new ArrayList<>();

		for( Move tmpMove : cachedMoves ) {

			LinearPosition tmpPos = (LinearPosition) tmpMove.getPosition();

			if( position.getPosition() == tmpPos.getPosition() )
				removeList.add( tmpMove );
		}

		cachedMoves.removeAll( removeList );

		if( log.isTraceEnabled() )
			log.trace( "New cached move list=" + cachedMoves );
	}

	@Override
	public List<Move> getNextMoves() {

		if( cachedMoves == null ) {

			preCacheMoves();
		}

		return cachedMoves;
	}

	private void preCacheMoves() {

		cachedMoves = new ArrayList<Move>( Board.NROFFIELDS );

		// First through all positions
		for( int position = 0; position < NROFFIELDS; position++ ) {

			// Check if first position is legal move
			if( getFields()[ position ] == 0 ) {

				LinearPosition linPos = new LinearPosition( position );

				// Create new legal move and add to list
				cachedMoves.add( new LinearMove( linPos ) );
			}
		}
	}

	public void setCachedMoves( List<Move> cachedMoves ) {

		this.cachedMoves = cachedMoves;
	}

	public List<Move> getCachedMoves() {

		return cachedMoves;
	}
}
