package ee.ut.joshi.connectsix.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import ee.ut.joshi.connectsix.model.linear.LinearBoard;
import ee.ut.joshi.connectsix.model.linear.LinearMove;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;

public class ConnectFourBoard extends LinearBoard {

	public ConnectFourBoard() {

		byte[] fields = new byte[ Board.NROFFIELDS ];

		// Initialize with empty fields
		for( int i = 0; i < fields.length; i++ ) {

			fields[ i ] = 0;
		}

		// Set top row to arbitrary value
		// so the board would be 6x7 board instead
		// TODO: Add to properties
		// for( int i = 0; i < Board.WIDTH; i++ ) {
		//
		// fields[ i ] = (byte) ( 255 - i );
		// }

		setFields( fields );
	}

	@Override
	public boolean isLegalMove( Move move ) {

		if( move == null )
			return false;

		// By default we assume that the move is not legal until proven otherwise
		boolean isLegal = false;

		// Check from next available moves if the move is included there
		for( Move legalMove : getNextMoves() ) {

			if( move.equals( legalMove ) )
				isLegal = true;
		}

		return isLegal;
	}

	@Override
	public List<Move> getNextMoves() {

		List<Move> moves = new ArrayList<Move>();

		// Check all columns, starting from left to right
		for( int column = 0; column < Board.WIDTH; column++ ) {

			Move move = checkColumn( column );

			if( move != null )
				moves.add( move );
		}

		return moves;
	}

	private Move checkColumn( int column ) {

		// Check all rows in one column, starting from bottom to top
		for( int row = Board.WIDTH - 1; row >= 0; row-- ) {

			int fIndx = row * Board.WIDTH + column;

			if( getFields()[ fIndx ] == Board.EMPTY )
				return new LinearMove( new LinearPosition( fIndx ) );
		}

		return null;
	}

	@Override
	public boolean equals( Object obj ) {

		if( obj instanceof ConnectFourBoard ) {

			ConnectFourBoard board = (ConnectFourBoard) obj;

			if( Arrays.equals( getFields(), board.getFields() ) )
				return true;
		}

		return false;
	}

	@Override
	public int hashCode() {

		return Arrays.hashCode( getFields() );
	}
}
