package ee.ut.joshi.connectsix.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.linear.LinearMove;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;

public class SmartGomokuBoard extends GomokuBoard {

	private static final Logger log = Logger.getLogger( SmartGomokuBoard.class );

	private static final int smartMovesDistance = ConnectObotProps.smartMoveDistance();

	private List<Move> smartMoves;

	public SmartGomokuBoard() {

		this( true );

		smartMoves = new ArrayList<>( Board.NROFFIELDS );

		// TODO: Add more, dynamic?
		if( Board.WIDTH == 7 ) {
			smartMoves.add( new LinearMove( new LinearPosition( Board.NROFFIELDS / 2 ) ) );

		} else if( Board.WIDTH == 10 ) {

			smartMoves.add( new LinearMove( new LinearPosition( 44 ) ) );
			smartMoves.add( new LinearMove( new LinearPosition( 45 ) ) );
			smartMoves.add( new LinearMove( new LinearPosition( 54 ) ) );
			smartMoves.add( new LinearMove( new LinearPosition( 55 ) ) );

		} else {

			smartMoves.add( new LinearMove( new LinearPosition( Board.NROFFIELDS / 2 ) ) );
		}
	}

	public SmartGomokuBoard( boolean init ) {

		if( init ) {

			byte[] fields = new byte[ Board.NROFFIELDS ];

			for( int i = 0; i < fields.length; i++ ) {

				fields[ i ] = 0;
			}

			setFields( fields );
		}
	}

	@Override
	public void makeMove( Player player, Move move ) {

		if( log.isTraceEnabled() ) {

			log.trace( "Making move=" + move + ";player=" + player );
			log.trace( "Current smart move list=" + smartMoves );
		}

		super.makeMove( player, move );

		smartMoves.remove( move );

		addSmartMoves( move );

		if( log.isTraceEnabled() )
			log.trace( "New smart move list=" + smartMoves );
	}

	@Override
	public List<Move> getNextMoves() {

		return smartMoves;
	}

	protected void addSmartMoves( Move move ) {

		// Check if we have correct type of move
		if( move instanceof LinearMove ) {

			// Extract position index
			LinearPosition linPos = (LinearPosition) move.getPosition();

			int position = linPos.getPosition();

			// Loop through all possible positions
			for( int fieldPos = 0; fieldPos < getFields().length; fieldPos++ ) {

				// Check if the position is viable/empty
				if( isViableMove( position, fieldPos ) ) {

					LinearMove newMove = new LinearMove( new LinearPosition( fieldPos ) );

					smartMoves.add( newMove );
				}
			}

		} else {

			log.fatal( "Move not supported" );
			System.exit( 1 );
		}
	}

	private boolean isViableMove( int position, int fieldPos ) {

		// Check if the position is viable/empty
		if( getFields()[ fieldPos ] != Board.EMPTY )
			return false;

		// Check the move distance (close one are more smart in most cases)
		if( distance( position, fieldPos ) > smartMovesDistance )
			return false;

		for( Move smartMove : smartMoves ) {

			LinearPosition linPos = (LinearPosition) smartMove.getPosition();

			if( linPos.getPosition() == fieldPos )
				return false;
		}

		return true;
	}

	private int distance( int posOne, int posTwo ) {

		// Naive way how to calculate distance between two points
		// Calculates only vertical and horizontal distance (no Pythagoras theorem)

		int maxDiff = 0;

		int posOneX = posOne % Board.WIDTH;
		int posTwoX = posTwo % Board.WIDTH;

		int posOneY = (int) Math.floor( posOne / Board.WIDTH );
		int posTwoY = (int) Math.floor( posTwo / Board.WIDTH );

		if( Math.abs( posOneX - posTwoX ) > maxDiff ) {

			maxDiff = Math.abs( posOneX - posTwoX );
		}

		if( Math.abs( posOneY - posTwoY ) > maxDiff ) {

			maxDiff = Math.abs( posOneY - posTwoY );
		}

		return maxDiff;
	}

	public void setSmartMoves( List<Move> smartMoves ) {

		this.smartMoves = smartMoves;
	}

	public List<Move> getSmartMoves() {

		return smartMoves;
	}
}
