package ee.ut.joshi.connectsix.model.linear;

import ee.ut.joshi.connectsix.model.DualMove;
import ee.ut.joshi.connectsix.model.Position;

/**
 * Game play move with two positions. Position values are considered to be with
 * linear type of the game board data, i.e. a single array of fields
 */
public class LinearDualMove implements DualMove {

	private final LinearPosition fst;
	private final LinearPosition snd;

	public LinearDualMove( LinearPosition fst, LinearPosition snd ) {

		this.fst = fst;
		this.snd = snd;
	}

	@Override
	public boolean equals( Object o ) {

		if( o instanceof LinearDualMove ) {

			LinearDualMove move = (LinearDualMove) o;

			LinearPosition posOne = move.getFstPos();
			LinearPosition posTwo = move.getSndPos();

			boolean isSame = fst.getPosition() == posOne.getPosition() && snd.getPosition() == posTwo.getPosition();
			boolean isSameRev = snd.getPosition() == posOne.getPosition() && fst.getPosition() == posTwo.getPosition();

			if( isSame || isSameRev )
				return true;
		}

		return false;
	}

	public LinearDualMove( Integer fst, Integer snd ) {

		this( new LinearPosition( fst ), new LinearPosition( snd ) );
	}

	@Override
	public Position getPosition() {

		return fst;
	}

	@Override
	public LinearPosition getFstPos() {

		return fst;
	}

	@Override
	public LinearPosition getSndPos() {

		return snd;
	}

	@Override
	public String toString() {

		return "{" + fst + ":" + snd + "}";
	}
}
