package ee.ut.joshi.connectsix.model.matrix;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.Position;

public class MatrixBoard implements Board<byte[][]> {

	private byte fields[][] = new byte[WIDTH][WIDTH];

	private Position winningPosition;

	private Move lastMove;

	public MatrixBoard() {

	}

	@Override
	public boolean isLegalMove( Move move ) {

		if( move instanceof MatrixMove ) {

			MatrixMove matrixMove = (MatrixMove) move;

			MatrixPosition matrixPos = matrixMove.getPosition();

			if( fields[ matrixPos.getRow() ][ matrixPos.getCol() ] == Board.EMPTY )
				return true;
		}

		return false;
	}

	public void makeMove( Player player, Move move ) {

		try {

			if( move instanceof MatrixDualMove ) {

				MatrixDualMove matrixMove = (MatrixDualMove) move;

				fields[ matrixMove.getFstPos().getRow() ][ matrixMove.getFstPos().getCol() ] = player.getValue();
				fields[ matrixMove.getSndPos().getRow() ][ matrixMove.getSndPos().getCol() ] = player.getValue();

			} else if( move instanceof MatrixMove ) {

				MatrixMove matrixMove = (MatrixMove) move;

				fields[ matrixMove.getPosition().getRow() ][ matrixMove.getPosition().getCol() ] = player.getValue();

			} else {

				throw new Exception( "Move not supported." );
			}

		} catch( Exception e ) {

			System.err.println( e.getMessage() );
			e.printStackTrace();
		}
	}

	public byte[][] getFields() {
		return fields;
	}

	public void setFields( byte[][] fields ) {
		this.fields = fields;
	}

	public List<Move> getNextMoves() {

		List<Move> moves = new ArrayList<Move>();

		// First loop
		for( int row = 0; row < WIDTH; row++ ) {
			for( int col = 0; col < WIDTH; col++ ) {

				// Check if first position is legal move
				if( fields[ row ][ col ] == 0 ) {

					// Second loop starting from current position
					for( int rowSnd = row; rowSnd < WIDTH; rowSnd++ ) {
						for( int colSnd = col; colSnd < WIDTH; colSnd++ ) {

							// Check if second position is legal move
							if( fields[ rowSnd ][ colSnd ] == 0 && ( row != rowSnd || col != colSnd ) ) {

								MatrixPosition fstPos = new MatrixPosition( row, col );
								MatrixPosition sndPos = new MatrixPosition( rowSnd, colSnd );

								// Create new legal move and add to list
								moves.add( new MatrixDualMove( fstPos, sndPos ) );
							}
						}
					}
				}
			}
		}

		return moves;
	}

	public boolean isGameEnd() {

		// TODO: Incorrect location for this check in case of last move is
		// winning move
		if( checkDraw() )
			return true;

		boolean result = false;

		for( int row = 0; row < WIDTH; row++ ) {
			for( int col = 0; col < WIDTH; col++ ) {

				result = ( result || checkHori( row, col ) );
				result = ( result || checkVert( row, col ) );
				result = ( result || checkDiag( row, col ) );
				result = ( result || checkDiagRev( row, col ) );

				if( result ) {

					winningPosition = new MatrixPosition( row, col );

					return true;
				}
			}
		}

		return false;
	}

	@Override
	public Move getLastMove() {

		return lastMove;
	}
	
	@Override
	public void setLastMove( Move lastMove ) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Position getWinningPosition() {

		return winningPosition;
	}

	@Override
	public byte getWinningPlayer() {
		// TODO Auto-generated method stub
		return 0;
	}

	private boolean checkDraw() {

		int nrOfFreeFields = 0;

		for( int row = 0; row < WIDTH; row++ ) {
			for( int col = 0; col < WIDTH; col++ ) {

				if( fields[ row ][ col ] == 0 )
					nrOfFreeFields++;

				if( nrOfFreeFields > 1 )
					return false;
			}
		}

		winningPosition = new MatrixPosition( WIDTH, WIDTH );

		return true;
	}

	private boolean checkHori( int row, int col ) {

		if( col >= WIDTH - RECURSION )
			return false;

		// Next check if first player has winning condition
		if( checkHoriRec( PLRONE, row, col, RECURSION ) )
			return true;

		// Next check if second player has winning condition
		if( checkHoriRec( PLRTWO, row, col, RECURSION ) )
			return true;

		return false;
	}

	private boolean checkHoriRec( byte player, int row, int col, int recursion ) {

		// If we find a field that is not for player, it's not a winning
		// condition
		if( fields[ row ][ col ] != player )
			return false;

		// Check if we have reached maximum recursion level
		// i.e. number of fields owned by player in a row
		if( recursion != 0 )
			return checkHoriRec( player, row, ++col, --recursion );

		// If we reached max level recursion and all fields so far are player
		// fields then it's a winning condition
		return true;
	}

	private boolean checkVert( int row, int col ) {

		if( row >= WIDTH - RECURSION )
			return false;

		// Next check if first player has winning condition
		if( checkVertRec( PLRONE, row, col, RECURSION ) )
			return true;

		// Next check if second player has winning condition
		if( checkVertRec( PLRTWO, row, col, RECURSION ) )
			return true;

		return false;
	}

	private boolean checkVertRec( byte player, int row, int col, int recursion ) {

		// If we find a field that is not for player, it's not a winning
		// condition
		if( fields[ row ][ col ] != player )
			return false;

		// Check if we have reached maximum recursion level
		// i.e. number of fields owned by player in a row
		if( recursion != 0 )
			return checkVertRec( player, ++row, col, --recursion );

		// If we reached max level recursion and all fields so far are player
		// fields
		// then it's a winning condition
		return true;
	}

	private boolean checkDiag( int row, int col ) {

		if( row >= WIDTH - RECURSION || col >= WIDTH - RECURSION )
			return false;

		// Next check if first player has winning condition
		if( checkDiagRec( PLRONE, row, col, RECURSION ) )
			return true;

		// Next check if second player has winning condition
		if( checkDiagRec( PLRTWO, row, col, RECURSION ) )
			return true;

		return false;
	}

	private boolean checkDiagRec( byte player, int row, int col, int recursion ) {

		// If we find a field that is not for player, it's not a winning
		// condition
		if( fields[ row ][ col ] != player )
			return false;

		// Check if we have reached maximum recursion level
		// i.e. number of fields owned by player in a row
		if( recursion != 0 )
			return checkDiagRec( player, ++row, ++col, --recursion );

		// If we reached max level recursion and all fields so far are player
		// fields
		// then it's a winning condition
		return true;
	}

	private boolean checkDiagRev( int row, int col ) {

		// TODO: Test this, was implemented later
		if( row < RECURSION || col >= WIDTH - RECURSION )
			return false;

		// Next check if first player has winning condition
		if( checkDiagRevRec( PLRONE, row, col, RECURSION ) )
			return true;

		// Next check if second player has winning condition
		if( checkDiagRevRec( PLRTWO, row, col, RECURSION ) )
			return true;

		return false;
	}

	private boolean checkDiagRevRec( byte player, int row, int col, int recursion ) {

		// If we find a field that is not for player, it's not a winning
		// condition
		if( fields[ row ][ col ] != player )
			return false;

		// Check if we have reached maximum recursion level
		// i.e. number of fields owned by player in a row
		if( recursion != 0 )
			// TODO: Test this, was implemented later
			return checkDiagRevRec( player, --row, ++col, --recursion );

		// If we reached max level recursion and all fields so far are player
		// fields
		// then it's a winning condition
		return true;
	}

	@Override
	public String toString() {

		String result = "";

		for( byte[] row : fields ) {
			result += Arrays.toString( row ) + "\n";
		}

		return result;
	}
}
