package ee.ut.joshi.connectsix.model.player;

import java.util.Scanner;

import org.apache.log4j.Logger;
import org.fusesource.jansi.Ansi;
import org.fusesource.jansi.AnsiConsole;

import ee.ut.joshi.connectsix.logic.board.ParallelBoardPrinter;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearMove;

public class HumanPlayer extends Player {

	private static Logger log = Logger.getLogger( HumanPlayer.class );

	// private int value = -1;

	public HumanPlayer( byte value, String name ) {
		super( value, name );

		AnsiConsole.systemInstall();
	}

	@Override
	public Move getNextMove( Board<? extends Object> board ) {

		log.debug( "Requesting next move for player " + this );

		System.out.println( "\nLast move was:" + board.getLastMove() );
		System.out.println( "\nBoard:" );
		printAnsiBoard( ParallelBoardPrinter.toStringWithMoves( board ), board );
		System.out.println( "Enter your move: " );

		Move nextMove;
		try {

			Scanner in = new Scanner( System.in );
			int move = in.nextInt();

			System.out.println( "Your move was: " + move );

			nextMove = new LinearMove( move );

			if( !board.isLegalMove( nextMove ) )
				throw new Exception( "Illegal move!" );

		} catch( Exception e ) {

			System.out.println( Ansi.ansi().render( "@|red,bold Incorrect input! Try again.|@" ) );
			System.out.println( e.getMessage() );

			return getNextMove( board );
		}

		return nextMove;
	}

	private void printAnsiBoard( String stringWithMoves, Board<? extends Object> board ) {

		String colored;

		if( getValue() == Board.PLRONE ) {

			colored = stringWithMoves.replace( "X", "@|bg_blue,bold,white X|@" );
			colored = colored.replace( "O", "@|bg_red,bold,white O|@" );

		} else {

			colored = stringWithMoves.replace( "X", "@|bg_red,bold,white X|@" );
			colored = colored.replace( "O", "@|bg_blue,bold,white O|@" );
		}

		colored = colored.replace( "#", "@|bg_green,bold,white #|@" );

		if( getValue() == Board.PLRONE )
			colored = colored.replace( "#", "O" );
		else if( getValue() == Board.PLRTWO )
			colored = colored.replace( "#", "X" );

		System.out.println( Ansi.ansi().render( colored ) );
	}

	@Override
	public String toString() {

		return "Human  :" + super.toString();
	}
}
