package ee.ut.joshi.connectsix.model.player;

import java.util.Map;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.gametree.NegamaxV3;
import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttribute;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributeDraw;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataAttributePlayer;
import ee.ut.joshi.connectsix.ai.randomforest.dataattr.DataEntry;
import ee.ut.joshi.connectsix.ai.randomforest.linear.LinearDataEntry;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearBoard;

public class MiniMaxPlayer extends Player implements Cloneable {

	private static final Logger log = Logger.getLogger( MiniMaxPlayer.class );

	private static final double winPrediction = ConnectObotProps.winPredictionWeight();
	private static final double drawPrediction = ConnectObotProps.drawPredictionWeight();
	private static final double lossPrediction = ConnectObotProps.lossPredictionWeight();

	private RandomForest forest;

	public MiniMaxPlayer( int value ) {

		super( value );
	}

	public MiniMaxPlayer( int value, RandomForest forest ) {

		this( value );
		this.forest = forest;
	}

	public Integer evaluateBoard( Board<? extends Object> board ) {

		// log.trace( "Evaluating board: " + board );

		Integer result = 0;

		DataAttributePlayer plrAttr = new DataAttributePlayer( Board.PLRONE );
		DataAttributeDraw drwAttr = new DataAttributeDraw();
		DataAttributePlayer oppAttr = new DataAttributePlayer( Board.PLRTWO );

		Integer win = 0;
		Integer draw = 0;
		Integer loss = 0;

		if( board instanceof LinearBoard ) {

			// TODO: Byte 0??
			DataEntry dataEntry = new LinearDataEntry( ( (LinearBoard) board ).getFields(), (byte) 0 );

			Map<DataAttribute, Integer> map = forest.findCategoryMap( dataEntry );

			// log.debug( "Random forest result map: " + map );
			// log.debug( "Random forest result map/draw: " + map.get( drwAttr ) );

			if( getValue() == Board.PLRONE ) {

				win = map.get( plrAttr );
				draw = map.get( drwAttr );
				loss = map.get( oppAttr );
			}

			if( getValue() == Board.PLRTWO ) {

				loss = map.get( plrAttr );
				draw = map.get( drwAttr );
				win = map.get( oppAttr );
			}

			win = win == null ? 0 : win;
			draw = draw == null ? 0 : draw;
			loss = loss == null ? 0 : loss;

			Integer total = win + draw + loss + 1;

			// Normalization of result into values of [0..100], ignoring failed predictions (nulls)
			result += new Double( ( win * 100 / total ) * winPrediction ).intValue();
			result += new Double( ( draw * 100 / total ) * drawPrediction ).intValue();
			result += new Double( ( loss * 100 / total ) * lossPrediction ).intValue();

		} else {

			log.fatal( "Board not supported" );
			System.exit( 1 );
		}
		//
		// log.debug( "Output from random tree: win=" + win + ";draw=" + draw + ";loss=" + loss );
		// log.debug( "Board evaluation: " + result );

		return result;
	}

	@Override
	public Move getNextMove( Board<? extends Object> board ) {

		log.trace( "Requesting new move on board: " + board );

		// First we make a clone from current player.
		// This is because this player will also evaluate board
		// in perspective of opponent with the knowledge behind
		// this player

		// Current player clone
		Player opponent = null;

		try {
			opponent = (Player) this.clone();

		} catch( CloneNotSupportedException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// We change the value of player mark up on the board,
		// but the knowledge remains
		byte newValue = opponent.getValue() == Board.PLRONE ? Board.PLRTWO : Board.PLRONE;

		opponent.setValue( newValue );

		if( getValue() == Board.PLRONE )
			return ( new NegamaxV3( this, opponent ) ).negamax( board );

		if( getValue() == Board.PLRTWO )
			return ( new NegamaxV3( opponent, this ) ).negamax( board );

		// And now find recursively best move with MiniMax algorithm
		return ( new NegamaxV3( this, opponent ) ).negamax( board );
	}

	public RandomForest getForest() {
		return forest;
	}

	public void setForest( RandomForest forest ) {
		this.forest = forest;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {

		MiniMaxPlayer clone = new MiniMaxPlayer( getValue() );
		clone.setForest( forest );

		return clone;
	}

	@Override
	public String toString() {

		return "MiniMax:" + super.toString();
	}
}
