package ee.ut.joshi.connectsix.model.player;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.gametree.Negamax;
import ee.ut.joshi.connectsix.ai.gametree.threaded.NegamaxManager;
import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.logic.factory.ReverseBoardFactory;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;

public class NegaMaxPlayer extends MiniMaxPlayer implements Cloneable {

	private static final Logger log = Logger.getLogger( NegaMaxPlayer.class );

	private boolean verbose = false;

	public NegaMaxPlayer( int value ) {

		super( value );
	}

	public NegaMaxPlayer( int value, RandomForest forest ) {

		super( value, forest );
	}

	public NegaMaxPlayer( int value, RandomForest forest, boolean verbose ) {

		super( value, forest );

		this.verbose = verbose;
	}

	@Override
	public Move getNextMove( Board<? extends Object> board ) {

		if( log.isTraceEnabled() )
			log.trace( "Requesting new move on board: " + board );

		// First we make a clone from current player.
		// This is because this player will also evaluate board
		// in perspective of opponent with the knowledge behind
		// this player

		// Current player clone
		Player plrOne = null;
		Player plrTwo = null;

		plrOne = this.clone();
		plrTwo = this.clone();

		if( getValue() == Board.PLRONE ) {

			plrOne.setValue( Board.PLRONE );
			plrTwo.setValue( Board.PLRTWO );

			Negamax negamax = NegamaxManager.instance().negamaxInstance( plrOne, plrTwo );

			if( verbose )
				negamax.printMoveScores();

			return negamax.negamax( board );

		} else {

			plrOne.setValue( Board.PLRONE );
			plrTwo.setValue( Board.PLRTWO );

			Negamax negamax = NegamaxManager.instance().negamaxInstance( plrOne, plrTwo );

			if( verbose )
				negamax.printMoveScores();

			Board<? extends Object> reversed = ReverseBoardFactory.reverseBoard( board );

			return negamax.negamax( reversed );
		}
	}

	@Override
	public NegaMaxPlayer clone() {

		NegaMaxPlayer clone = new NegaMaxPlayer( getValue() );
		clone.setForest( getForest() );
		clone.setName( getName() );

		return clone;
	}

	@Override
	public String toString() {

		return "NegaMax-" + getName();
	}
}
