package ee.ut.joshi.connectsix.model.player;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.logic.factory.BoardCloneFactory;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearBoard;

public class ScoreBasedPlayer extends Player {

	private static final Logger log = Logger.getLogger( ScoreBasedPlayer.class );

	public ScoreBasedPlayer( byte value ) {

		super( value );
	}

	public ScoreBasedPlayer( byte value, String name ) {

		super( value, name );
	}

	public ScoreBasedPlayer( int value ) {

		super( value );
	}

	@Override
	public Move getNextMove( Board<? extends Object> board ) {

		int bestScore = 0;
		Move bestMove = board.getNextMoves().get( 0 );

		for( Move move : board.getNextMoves() ) {

			Board<? extends Object> clone = BoardCloneFactory.cloneBoard( board );

			if( board instanceof LinearBoard ) {

				LinearBoard linBoard = (LinearBoard) clone;

				linBoard.makeMove( this, move );

				int score = linBoard.getBoardScore();

				log.trace( "Score for move=" + move + " is " + score );

				if( getValue() == Board.PLRONE & score > bestScore ) {

					bestScore = score;
					bestMove = move;
				}

				if( getValue() == Board.PLRTWO & score < bestScore ) {

					bestScore = score;
					bestMove = move;
				}

			} else {

				log.fatal( "Board not supported!" );
				System.exit( 1 );
			}

		}

		return bestMove;
	}
}
