package ee.ut.joshi.connectsix.ui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.PropertyConfigurator;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.logic.factory.RandomForestParserXML;
import ee.ut.joshi.connectsix.main.ConnectObotProps;

public class ConnectSixGui {

	public static void main( String[] args ) throws FileNotFoundException, IOException, ParserConfigurationException, SAXException {
		
		PropertyConfigurator.configure( "etc/log4j-gui.properties" );
		ConnectObotProps props = new ConnectObotProps();
		props.load( new FileInputStream( "etc/connect-six.properties" ) );

		String latest = findLatestForest();
		
		InputStream io;

		try {
			io = new FileInputStream( "log/forests/" + latest );
		} catch( FileNotFoundException e ) {

			System.out.println( "File not found: " + args[ 0 ] );
			return;
		}
		
		System.out.println( "Forest file=" + latest );

		RandomForest forest;
		RandomForestParserXML parser = new RandomForestParserXML( io );

		forest = parser.parseForest();
		
		ForestVisualizer gui = new ForestVisualizer( forest );
		
		gui.setVisible( true );
	}
	

	private static String findLatestForest() {

		File dir = new File( "log/forests" );

		String[] children = dir.list();

		FilenameFilter filter = new FilenameFilter() {
			public boolean accept( File dir, String name ) {
				return name.startsWith( "forests_iter_" );
			}
		};
		children = dir.list( filter );

		return children[ children.length - 1 ];
	}
}
