package ee.ut.joshi.connectsix.ui.game;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearBoard;
import ee.ut.joshi.connectsix.model.player.GuiPlayer;

public class GameBoardGui extends JFrame {

	/**  */
	private static final long serialVersionUID = 1L;

	private JPanel topPanel;
	private JPanel midPanel;
	private JPanel botPanel;

	private JLabel actingPlayer;
	private JLabel botInfoText;

	private GameBoard gameBoard;

	private GuiGameController game;

	public GameBoardGui( Player plrOne, Player plrTwo ) {

		game = new GuiGameController( this, plrOne, plrTwo );

		setTitle( "Game board" );
		// setSize( 600, 800 );
		setLocation( 250, 250 );
		setDefaultCloseOperation( EXIT_ON_CLOSE );

		setLayout( new BorderLayout() );

		topPanel = new JPanel();
		topPanel.setAlignmentX( Component.CENTER_ALIGNMENT );
		topPanel.setLayout( new GridLayout( 0, 1 ) );
		topPanel.add( new JLabel( "Acting player is" ) );

		actingPlayer = new JLabel( plrOne.toString() );
		topPanel.add( actingPlayer );

		midPanel = new JPanel();
		gameBoard = new GameBoard();
		midPanel.add( gameBoard );

		botPanel = new JPanel();
		botPanel.setAlignmentX( Component.CENTER_ALIGNMENT );

		botInfoText = new JLabel( "            " );
		botPanel.add( botInfoText );

		add( topPanel, BorderLayout.PAGE_START );
		add( midPanel, BorderLayout.CENTER );
		add( botPanel, BorderLayout.PAGE_END );

		pack();

	}

	public void startGame() {

		game.startGame();
	}

	public void updateState() {

		actingPlayer.setText( game.getActivePlayer().getName() );

		if( game.getActivePlayer() instanceof GuiPlayer )
			botInfoText.setText( "Make your move human." );
		else
			botInfoText.setText( "Thinking..." );

		if( game.getBoard().isGameEnd() )
			botInfoText.setText( "Game over!" );
		else
			actingPlayer.setText( game.getActivePlayer().toString() );

		gameBoard.updateFields( game.getBoard() );
	}

	public GuiGameController getGame() {
		return game;
	}

	public void setGame( GuiGameController game ) {
		this.game = game;
	}

	private class GameBoard extends JPanel {

		private static final long serialVersionUID = 1L;

		private List<JButton> fields = new ArrayList<>();

		public GameBoard() {

			setPreferredSize( new Dimension( Board.WIDTH * 20, Board.WIDTH * 20 ) );
			setLayout( new GridLayout( Board.WIDTH, Board.WIDTH, 0, 0 ) );

			for( int i = 0; i < Board.NROFFIELDS; i++ ) {

				JButton field = new JButton( " " );
				field.setActionCommand( "" + i );
				field.addActionListener( game );
				field.setMargin( new Insets( 0, 0, 0, 0 ) );

				add( field );

				fields.add( field );
			}
		}

		private void updateFields( Board<? extends Object> board ) {

			if( board instanceof LinearBoard ) {

				LinearBoard linBoard = (LinearBoard) board;

				for( int i = 0; i < fields.size(); i++ ) {

					if( linBoard.getFields()[ i ] == Board.PLRONE ) {

						fields.get( i ).setText( "X" );
						fields.get( i ).setForeground( Color.red );
					}

					if( linBoard.getFields()[ i ] == Board.PLRTWO ) {

						fields.get( i ).setText( "O" );
						fields.get( i ).setForeground( Color.blue );
					}

					if( linBoard.getFields()[ i ] == Board.EMPTY )
						fields.get( i ).setText( " " );

				}
			}
		}
	}
}
