package ee.ut.joshi.connectsix.ui.game;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.ai.gametree.threaded.NegamaxManager;
import ee.ut.joshi.connectsix.logic.Competition;
import ee.ut.joshi.connectsix.logic.GameController;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Move;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.linear.LinearMove;
import ee.ut.joshi.connectsix.model.linear.LinearPosition;
import ee.ut.joshi.connectsix.model.player.DrawPlayer;
import ee.ut.joshi.connectsix.model.player.GuiPlayer;

public class GuiGameController extends GameController implements ActionListener {

	private static final Logger log = Logger.getLogger( GuiGameController.class );

	GameBoardGui gameBoardGui;

	public GuiGameController( GameBoardGui gameBoardGui, Player plrOne, Player plrTwo ) {

		this.gameBoardGui = gameBoardGui;

		setPlayerOne( plrOne );
		setPlayerTwo( plrTwo );

		setActivePlayer( plrOne );
		setBoard( GuiProperties.instance().getBoardInstance() );

		// Quick hack so Negamax would use all threads
		NegamaxManager.instance().update( new Competition( null ), 1 );
	}

	@Override
	public Player startGame() {

		requestMove();

		return null;
	}

	@Override
	protected void makeMove( Move move ) {

		log.info( "Making move=" + move + ";activeplayer=" + getActivePlayer() );

		if( !getBoard().isLegalMove( move ) )
			return;

		super.makeMove( move );

		gameBoardGui.updateState();

		requestMove();
	}

	private void requestMove() {

		log.info( "Requesting move for activeplayer=" + getActivePlayer() );

		if( getBoard().isGameEnd() ) {

			gameEnd();
			return;
		}

		// If player is not GuiPlayer/Human, then request next move as per regular
		if( !( getActivePlayer() instanceof GuiPlayer ) ) {

			log.info( "Player not GuiPlayer, requesting move" );

			Move move = getActivePlayer().getNextMove( getBoard() );
			makeMove( move );
		}
	}

	private void gameEnd() {

		log.info( "Game ended!" );

		Byte win = getBoard().getWinningPlayer();

		setWinner( win.compareTo( getPlayerOne().getValue() ) == 0 ? getPlayerOne() : getPlayerTwo() );
		setWinner( win.compareTo( Board.EMPTY ) == 0 ? new DrawPlayer() : getWinner() );

		Logger.getLogger( "finalstates" ).trace(
				"Finished game between " + getPlayerOne() + " and " + getPlayerTwo() + ", winner=" + getWinner() );
		Logger.getLogger( "finalstates" ).trace( "Winning position: " + getBoard().getWinningPosition() );
		Logger.getLogger( "finalstates" ).trace( "Winner board: " + getBoard() );

		gameBoardGui.updateState();
	}

	@Override
	public void actionPerformed( ActionEvent e ) {

		if( getBoard().isGameEnd() )
			return;

		if( !( getActivePlayer() instanceof GuiPlayer ) && getActivePlayer() != e.getSource() ) {

			log.info( "Gui player tried to make turn while not his/her turn!" );

			return;
		}

		Integer indx = Integer.valueOf( e.getActionCommand() );

		log.info( "Player chosen move=" + e.getActionCommand() );

		Move move = new LinearMove( new LinearPosition( indx ) );

		makeMove( move );
	}
}
