package ee.ut.joshi.connectsix.ui.game;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import ee.ut.joshi.connectsix.logic.factory.BoardCloneFactory;
import ee.ut.joshi.connectsix.main.ConnectObotProps;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Connect6Board;
import ee.ut.joshi.connectsix.model.ConnectFourBoard;
import ee.ut.joshi.connectsix.model.SmartGomokuBoard;
import ee.ut.joshi.connectsix.model.TicBoard;

public class GuiProperties {

	private static final Logger log = Logger.getLogger( GuiProperties.class );

	public static final byte TICTACTOE = 1;
	public static final byte CONNECTFOUR = 2;
	public static final byte GOMOKU7 = 4;
	public static final byte GOMOKU10 = 8;
	public static final byte CONNECTSIX = 16;

	public static final String TICTACTOE_NAME = "TicTacToe";
	public static final String CONNECTFOUR_NAME = "Connect Four";
	public static final String GOMOKU7_NAME = "Gomoku 7x7";
	public static final String GOMOKU10_NAME = "Gomoku 10x10";
	public static final String CONNECTSIX_NAME = "Connect Six";

	public static final String TICTACTOE_PROPS = "gui/tictactoe.properties";
	public static final String CONNECTFOUR_PROPS = "gui/connect-four.properties";
	public static final String GOMOKU7_PROPS = "gui/gomoku7.properties";
	public static final String GOMOKU10_PROPS = "gui/gomoku10.properties";
	public static final String CONNECTSIX_PROPS = "gui/connect-six.properties";

	public static final String TICTACTOE_FORESTPATH = "gui/tictactoe/";
	public static final String CONNECTFOUR_FORESTPATH = "gui/connectfour/";
	public static final String GOMOKU7_FORESTPATH = "gui/gomoku7/";
	public static final String GOMOKU10_FORESTPATH = "gui/gomoku10/";
	public static final String CONNECTSIX_FORESTPATH = "gui/connectsix/";

	private byte activeGametype = GOMOKU10;
	private String activeName = GOMOKU10_NAME;
	private String activeProps = GOMOKU10_PROPS;
	private String activeForestpath = GOMOKU10_FORESTPATH;

	private Map<Byte, String> nameMap = new HashMap<>();
	private Map<Byte, String> propsMap = new HashMap<>();
	private Map<Byte, String> forestpathMap = new HashMap<>();
	private Map<Byte, Board<? extends Object>> boardMap = new HashMap<>();

	/** Configured game type with default as Gomoku 10x10 */

	private static final GuiProperties instance = new GuiProperties();

	private GuiProperties() {

		nameMap.put( TICTACTOE, TICTACTOE_NAME );
		nameMap.put( CONNECTFOUR, CONNECTFOUR_NAME );
		nameMap.put( GOMOKU7, GOMOKU7_NAME );
		nameMap.put( GOMOKU10, GOMOKU10_NAME );
		nameMap.put( CONNECTSIX, CONNECTSIX_NAME );

		propsMap.put( TICTACTOE, TICTACTOE_PROPS );
		propsMap.put( CONNECTFOUR, CONNECTFOUR_PROPS );
		propsMap.put( GOMOKU7, GOMOKU7_PROPS );
		propsMap.put( GOMOKU10, GOMOKU10_PROPS );
		propsMap.put( CONNECTSIX, CONNECTSIX_PROPS );

		forestpathMap.put( TICTACTOE, TICTACTOE_FORESTPATH );
		forestpathMap.put( CONNECTFOUR, CONNECTFOUR_FORESTPATH );
		forestpathMap.put( GOMOKU7, GOMOKU7_FORESTPATH );
		forestpathMap.put( GOMOKU10, GOMOKU10_FORESTPATH );
		forestpathMap.put( CONNECTSIX, CONNECTSIX_FORESTPATH );
	}

	public static GuiProperties instance() {

		return instance;
	}

	public Board<? extends Object> getBoardInstance() {

		Board<? extends Object> board = boardMap.get( activeGametype );

		if( board != null )
			return BoardCloneFactory.cloneBoard( board );

		log.fatal( "Game type not supported!" );

		return null;
	}

	public byte getGametype() {
		return activeGametype;
	}

	public void setGametype( byte gametype ) {

		log.info( "Setting game type to: " + nameMap.get( gametype ) );

		ConnectObotProps props = new ConnectObotProps();

		try {

			props.load( new FileInputStream( propsMap.get( gametype ) ) );

		} catch( FileNotFoundException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch( IOException e ) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		boardMap.put( TICTACTOE, new TicBoard() );
		boardMap.put( CONNECTFOUR, new ConnectFourBoard() );
		boardMap.put( GOMOKU7, new SmartGomokuBoard() );
		boardMap.put( GOMOKU10, new SmartGomokuBoard() );
		boardMap.put( CONNECTSIX, new Connect6Board() );

		activeGametype = gametype;
		activeName = nameMap.get( gametype );
		activeProps = propsMap.get( gametype );
		activeForestpath = forestpathMap.get( gametype );

		this.activeGametype = gametype;
	}

	public byte getActiveGametype() {
		return activeGametype;
	}

	public String getActiveName() {
		return activeName;
	}

	public String getActiveProps() {
		return activeProps;
	}

	public String getActiveForestpath() {
		return activeForestpath;
	}
}
