package ee.ut.joshi.connectsix.ui.game;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import ee.ut.joshi.connectsix.ai.randomforest.RandomForest;
import ee.ut.joshi.connectsix.logic.factory.RandomForestParserXML;
import ee.ut.joshi.connectsix.model.Board;
import ee.ut.joshi.connectsix.model.Player;
import ee.ut.joshi.connectsix.model.player.GuiNegamaxPlayer;
import ee.ut.joshi.connectsix.model.player.GuiPlayer;

public class MainMenu extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	private static final Logger log = Logger.getLogger( MainMenu.class );

	private JButton pVp;
	private JButton pVai;
	private JButton aiVp;
	private JButton aiVai;

	private JTextField forestVSplr;
	private JTextField forestVSaiOne;
	private JTextField forestVSaiTwo;

	private GameTypePanel gameTypePanel = new GameTypePanel();

	public MainMenu() {

		setTitle( "Main menu" );
		setSize( 280, 400 );
		setResizable( false );
		setLocation( 200, 200 );
		setDefaultCloseOperation( EXIT_ON_CLOSE );

		JPanel buttonPanel = new JPanel();

		buttonPanel.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );

		buttonPanel.setLayout( new GridLayout( 0, 1, 10, 10 ) );

		// Game type selection
		buttonPanel.add( gameTypePanel );

		// Player selection
		pVp = new JButton( "Player VS Player" );
		pVp.setActionCommand( pVp.getText() );
		pVp.addActionListener( this );

		pVai = new JButton( "Player VS AI" );
		pVai.setActionCommand( pVai.getText() );
		pVai.addActionListener( this );

		aiVp = new JButton( "AI VS Player" );
		aiVp.setActionCommand( aiVp.getText() );
		aiVp.addActionListener( this );

		aiVai = new JButton( "AI VS AI" );
		aiVai.setActionCommand( aiVai.getText() );
		aiVai.addActionListener( this );

		forestVSplr = new JTextField( "forest01.xml" );
		forestVSaiOne = new JTextField( "forest01.xml" );
		forestVSaiTwo = new JTextField( "forest01.xml" );

		JPanel textPanel = new JPanel();
		textPanel.setLayout( new GridLayout( 1, 0, 10, 10 ) );
		textPanel.setBorder( new EmptyBorder( 0, 10, 25, 10 ) );

		textPanel.add( forestVSplr );

		JPanel dualTextPanel = new JPanel();
		dualTextPanel.setLayout( new GridLayout( 0, 1, 10, 3 ) );
		dualTextPanel.setBorder( new EmptyBorder( 0, 10, 0, 10 ) );

		dualTextPanel.add( forestVSaiOne );
		dualTextPanel.add( forestVSaiTwo );

		buttonPanel.add( pVp );
		buttonPanel.add( pVai );
		buttonPanel.add( aiVp );
		buttonPanel.add( textPanel );
		buttonPanel.add( aiVai );
		buttonPanel.add( dualTextPanel );

		add( buttonPanel );
	}

	@Override
	public void actionPerformed( ActionEvent e ) {

		GuiProperties.instance().setGametype( gameTypePanel.getGameType() );

		GameBoardGui gameboard = null;

		if( e.getActionCommand().equals( pVp.getText() ) ) {

			Player plrOne = new GuiPlayer( Board.PLRONE, "Player One" );
			Player plrTwo = new GuiPlayer( Board.PLRTWO, "Player Two" );

			gameboard = new GameBoardGui( plrOne, plrTwo );
		}

		if( e.getActionCommand().equals( aiVp.getText() ) ) {

			GuiNegamaxPlayer plrOne = new GuiNegamaxPlayer( Board.PLRONE, loadForest( forestVSplr.getText() ), true );
			Player plrTwo = new GuiPlayer( Board.PLRTWO, "Human player" );

			gameboard = new GameBoardGui( plrOne, plrTwo );

			plrOne.setGame( gameboard.getGame() );
		}

		if( e.getActionCommand().equals( pVai.getText() ) ) {

			Player plrOne = new GuiPlayer( Board.PLRONE, "Human player" );
			GuiNegamaxPlayer plrTwo = new GuiNegamaxPlayer( Board.PLRTWO, loadForest( forestVSplr.getText() ), true );

			gameboard = new GameBoardGui( plrOne, plrTwo );

			plrTwo.setGame( gameboard.getGame() );
		}

		if( e.getActionCommand().equals( aiVai.getText() ) ) {

			GuiNegamaxPlayer plrOne = new GuiNegamaxPlayer( Board.PLRONE, loadForest( forestVSaiOne.getText() ), true );
			GuiNegamaxPlayer plrTwo = new GuiNegamaxPlayer( Board.PLRTWO, loadForest( forestVSaiTwo.getText() ), true );

			gameboard = new GameBoardGui( plrOne, plrTwo );

			plrOne.setGame( gameboard.getGame() );
			plrTwo.setGame( gameboard.getGame() );
		}

		this.setVisible( false );
		gameboard.setVisible( true );
		
		gameboard.startGame();
	}

	private RandomForest loadForest( String filename ) {

		String file = GuiProperties.instance().getActiveForestpath() + filename;

		InputStream io;

		RandomForest forest = null;
		RandomForestParserXML parser;

		try {
			io = new FileInputStream( file );
			parser = new RandomForestParserXML( io );
			forest = parser.parseForest();

			log.info( "Forest file=" + file );

		} catch( FileNotFoundException e ) {

			log.fatal( "File not found: " + file );
		} catch( ParserConfigurationException | SAXException | IOException e ) {

			log.fatal( "Forest parse exception: " + e.getMessage() );
		}

		return forest;
	}
}
